@@ -52,9 +52,9 @@ struct UPlugData {
   UPlugEntrypoint  *entrypoint; /**< plugin entrypoint */
   uint32_t structSize;    /**< initialized to the size of this structure */
   uint32_t token;         /**< must be U_PLUG_TOKEN */
-  void *lib;              /**< plugin library, or NULL */
+  void *lib;              /**< plugin library, or nullptr */
   char libName[UPLUG_NAME_MAX];   /**< library name */
-  char sym[UPLUG_NAME_MAX];        /**< plugin symbol, or NULL */
+  char sym[UPLUG_NAME_MAX];        /**< plugin symbol, or nullptr */
   char config[UPLUG_NAME_MAX];     /**< configuration data */
   void *context;          /**< user context data */
   char name[UPLUG_NAME_MAX];   /**< name of plugin */
@@ -148,9 +148,9 @@ static int32_t searchForLibrary(void *lib) {
 U_CAPI char * U_EXPORT2
 uplug_findLibrary(void *lib, UErrorCode *status) {
   int32_t libEnt;
-  char *ret = NULL;
+  char *ret = nullptr;
   if(U_FAILURE(*status)) {
-    return NULL;
+    return nullptr;
   }
   libEnt = searchForLibrary(lib);
   if(libEnt!=-1) { 
@@ -164,9 +164,9 @@ uplug_findLibrary(void *lib, UErrorCode *status) {
 U_CAPI void * U_EXPORT2
 uplug_openLibrary(const char *libName, UErrorCode *status) {
   int32_t libEntry = -1;
-  void *lib = NULL;
+  void *lib = nullptr;
     
-  if(U_FAILURE(*status)) return NULL;
+  if(U_FAILURE(*status)) return nullptr;
 
   libEntry = searchForLibraryName(libName);
   if(libEntry == -1) {
@@ -177,7 +177,7 @@ uplug_openLibrary(const char *libName, UErrorCode *status) {
 #if UPLUG_TRACE
       DBG((stderr, "uplug_openLibrary() - out of library slots (max %d)\n", libraryMax));
 #endif
-      return NULL;
+      return nullptr;
     }
     /* Some operating systems don't want 
        DL operations from multiple threads. */
@@ -186,9 +186,9 @@ uplug_openLibrary(const char *libName, UErrorCode *status) {
     DBG((stderr, "uplug_openLibrary(%s,%s) libEntry %d, lib %p\n", libName, u_errorName(*status), libEntry, lib));
 #endif
         
-    if(libraryList[libEntry].lib == NULL || U_FAILURE(*status)) {
+    if(libraryList[libEntry].lib == nullptr || U_FAILURE(*status)) {
       /* cleanup. */
-      libraryList[libEntry].lib = NULL; /* failure with open */
+      libraryList[libEntry].lib = nullptr; /* failure with open */
       libraryList[libEntry].name[0] = 0;
 #if UPLUG_TRACE
       DBG((stderr, "uplug_openLibrary(%s,%s) libEntry %d, lib %p\n", libName, u_errorName(*status), libEntry, lib));
@@ -252,14 +252,14 @@ static int32_t uplug_pluginNumber(UPlugData* d) {
 
 U_CAPI UPlugData * U_EXPORT2
 uplug_nextPlug(UPlugData *prior) {
-  if(prior==NULL) {
+  if(prior==nullptr) {
     return pluginList;
   } else {
     UPlugData *nextPlug = &prior[1];
     UPlugData *pastPlug = &pluginList[pluginCount];
     
     if(nextPlug>=pastPlug) {
-      return NULL;
+      return nullptr;
     } else {
       return nextPlug;
     }
@@ -273,7 +273,7 @@ uplug_nextPlug(UPlugData *prior) {
  */
 static void uplug_callPlug(UPlugData *plug, UPlugReason reason, UErrorCode *status) {
   UPlugTokenReturn token;
-  if(plug==NULL||U_FAILURE(*status)) {
+  if(plug==nullptr||U_FAILURE(*status)) {
     return;
   }
   token = (*(plug->entrypoint))(plug, reason, status);
@@ -330,15 +330,15 @@ static void uplug_loadPlug(UPlugData *plug, UErrorCode *status) {
 
 static UPlugData *uplug_allocateEmptyPlug(UErrorCode *status)
 {
-  UPlugData *plug = NULL;
+  UPlugData *plug = nullptr;
 
   if(U_FAILURE(*status)) {
-    return NULL;
+    return nullptr;
   }
 
   if(pluginCount == UPLUG_PLUGIN_INITIAL_COUNT) {
     *status = U_MEMORY_ALLOCATION_ERROR;
-    return NULL;
+    return nullptr;
   }
 
   plug = &pluginList[pluginCount++];
@@ -353,8 +353,8 @@ static UPlugData *uplug_allocateEmptyPlug(UErrorCode *status)
   plug->libName[0] = 0;
   plug->config[0]=0;
   plug->sym[0]=0;
-  plug->lib=NULL;
-  plug->entrypoint=NULL;
+  plug->lib=nullptr;
+  plug->entrypoint=nullptr;
 
 
   return plug;
@@ -364,16 +364,16 @@ static UPlugData *uplug_allocatePlug(UPlugEntrypoint *entrypoint, const char *co
                                      UErrorCode *status) {
   UPlugData *plug = uplug_allocateEmptyPlug(status);
   if(U_FAILURE(*status)) {
-    return NULL;
+    return nullptr;
   }
 
-  if(config!=NULL) {
+  if(config!=nullptr) {
     uprv_strncpy(plug->config, config, UPLUG_NAME_MAX);
   } else {
     plug->config[0] = 0;
   }
     
-  if(symName!=NULL) {
+  if(symName!=nullptr) {
     uprv_strncpy(plug->sym, symName, UPLUG_NAME_MAX);
   } else {
     plug->sym[0] = 0;
@@ -393,7 +393,7 @@ static void uplug_deallocatePlug(UPlugData *plug, UErrorCode *status) {
     uplug_closeLibrary(plug->lib, &subStatus);
 #endif
   }
-  plug->lib = NULL;
+  plug->lib = nullptr;
   if(U_SUCCESS(*status) && U_FAILURE(subStatus)) {
     *status = subStatus;
   }
@@ -410,7 +410,7 @@ static void uplug_deallocatePlug(UPlugData *plug, UErrorCode *status) {
 }
 
 static void uplug_doUnloadPlug(UPlugData *plugToRemove, UErrorCode *status) {
-  if(plugToRemove != NULL) {
+  if(plugToRemove != nullptr) {
     uplug_unloadPlug(plugToRemove, status);
     uplug_deallocatePlug(plugToRemove, status);
   }
@@ -418,14 +418,14 @@ static void uplug_doUnloadPlug(UPlugData *plugToRemove, UErrorCode *status) {
 
 U_CAPI void U_EXPORT2
 uplug_removePlug(UPlugData *plug, UErrorCode *status)  {
-  UPlugData *cursor = NULL;
-  UPlugData *plugToRemove = NULL;
+  UPlugData *cursor = nullptr;
+  UPlugData *plugToRemove = nullptr;
   if(U_FAILURE(*status)) return;
     
-  for(cursor=pluginList;cursor!=NULL;) {
+  for(cursor=pluginList;cursor!=nullptr;) {
     if(cursor==plug) {
       plugToRemove = plug;
-      cursor=NULL;
+      cursor=nullptr;
     } else {
       cursor = uplug_nextPlug(cursor);
     }
@@ -481,7 +481,7 @@ uplug_getLibraryName(UPlugData *data, UErrorCode *status) {
 #if U_ENABLE_DYLOAD
     return uplug_findLibrary(data->lib, status);
 #else
-    return NULL;
+    return nullptr;
 #endif
   }
 }
@@ -510,7 +510,7 @@ uplug_getConfiguration(UPlugData *data) {
 U_CAPI UPlugData* U_EXPORT2
 uplug_getPlugInternal(int32_t n) { 
   if(n <0 || n >= pluginCount) {
-    return NULL;
+    return nullptr;
   } else { 
     return &(pluginList[n]);
   }
@@ -530,7 +530,7 @@ uplug_getPlugLoadStatus(UPlugData *plug) {
  */
 static UPlugData* uplug_initPlugFromEntrypointAndLibrary(UPlugEntrypoint *entrypoint, const char *config, void *lib, const char *sym,
                                                          UErrorCode *status) {
-  UPlugData *plug = NULL;
+  UPlugData *plug = nullptr;
 
   plug = uplug_allocatePlug(entrypoint, config, lib, sym, status);
 
@@ -538,13 +538,13 @@ static UPlugData* uplug_initPlugFromEntrypointAndLibrary(UPlugEntrypoint *entryp
     return plug;
   } else {
     uplug_deallocatePlug(plug, status);
-    return NULL;
+    return nullptr;
   }
 }
 
 U_CAPI UPlugData* U_EXPORT2
 uplug_loadPlugFromEntrypoint(UPlugEntrypoint *entrypoint, const char *config, UErrorCode *status) {
-  UPlugData* plug = uplug_initPlugFromEntrypointAndLibrary(entrypoint, config, NULL, NULL, status);
+  UPlugData* plug = uplug_initPlugFromEntrypointAndLibrary(entrypoint, config, nullptr, nullptr, status);
   uplug_loadPlug(plug, status);
   return plug;
 }
@@ -555,25 +555,25 @@ static UPlugData*
 uplug_initErrorPlug(const char *libName, const char *sym, const char *config, const char *nameOrError, UErrorCode loadStatus, UErrorCode *status)
 {
   UPlugData *plug = uplug_allocateEmptyPlug(status);
-  if(U_FAILURE(*status)) return NULL;
+  if(U_FAILURE(*status)) return nullptr;
 
   plug->pluginStatus = loadStatus;
   plug->awaitingLoad = false; /* Won't load. */
   plug->dontUnload = true; /* cannot unload. */
 
-  if(sym!=NULL) {
+  if(sym!=nullptr) {
     uprv_strncpy(plug->sym, sym, UPLUG_NAME_MAX);
   }
 
-  if(libName!=NULL) {
+  if(libName!=nullptr) {
     uprv_strncpy(plug->libName, libName, UPLUG_NAME_MAX);
   }
 
-  if(nameOrError!=NULL) {
+  if(nameOrError!=nullptr) {
     uprv_strncpy(plug->name, nameOrError, UPLUG_NAME_MAX);
   }
 
-  if(config!=NULL) {
+  if(config!=nullptr) {
     uprv_strncpy(plug->config, config, UPLUG_NAME_MAX);
   }
 
@@ -585,39 +585,39 @@ uplug_initErrorPlug(const char *libName, const char *sym, const char *config, co
  */
 static UPlugData* 
 uplug_initPlugFromLibrary(const char *libName, const char *sym, const char *config, UErrorCode *status) {
-  void *lib = NULL;
-  UPlugData *plug = NULL;
-  if(U_FAILURE(*status)) { return NULL; }
+  void *lib = nullptr;
+  UPlugData *plug = nullptr;
+  if(U_FAILURE(*status)) { return nullptr; }
   lib = uplug_openLibrary(libName, status);
-  if(lib!=NULL && U_SUCCESS(*status)) {
-    UPlugEntrypoint *entrypoint = NULL;
+  if(lib!=nullptr && U_SUCCESS(*status)) {
+    UPlugEntrypoint *entrypoint = nullptr;
     entrypoint = (UPlugEntrypoint*)uprv_dlsym_func(lib, sym, status);
 
-    if(entrypoint!=NULL&&U_SUCCESS(*status)) {
+    if(entrypoint!=nullptr&&U_SUCCESS(*status)) {
       plug = uplug_initPlugFromEntrypointAndLibrary(entrypoint, config, lib, sym, status);
-      if(plug!=NULL&&U_SUCCESS(*status)) {
+      if(plug!=nullptr&&U_SUCCESS(*status)) {
         plug->lib = lib; /* plug takes ownership of library */
-        lib = NULL; /* library is now owned by plugin. */
+        lib = nullptr; /* library is now owned by plugin. */
       }
     } else {
       UErrorCode subStatus = U_ZERO_ERROR;
-      plug = uplug_initErrorPlug(libName,sym,config,"ERROR: Could not load entrypoint",(lib==NULL)?U_MISSING_RESOURCE_ERROR:*status,&subStatus);
+      plug = uplug_initErrorPlug(libName,sym,config,"ERROR: Could not load entrypoint",(lib==nullptr)?U_MISSING_RESOURCE_ERROR:*status,&subStatus);
     }
-    if(lib!=NULL) { /* still need to close the lib */
+    if(lib!=nullptr) { /* still need to close the lib */
       UErrorCode subStatus = U_ZERO_ERROR;
       uplug_closeLibrary(lib, &subStatus); /* don't care here */
     }
   } else {
     UErrorCode subStatus = U_ZERO_ERROR;
-    plug = uplug_initErrorPlug(libName,sym,config,"ERROR: could not load library",(lib==NULL)?U_MISSING_RESOURCE_ERROR:*status,&subStatus);
+    plug = uplug_initErrorPlug(libName,sym,config,"ERROR: could not load library",(lib==nullptr)?U_MISSING_RESOURCE_ERROR:*status,&subStatus);
   }
   return plug;
 }
 
 U_CAPI UPlugData* U_EXPORT2
 uplug_loadPlugFromLibrary(const char *libName, const char *sym, const char *config, UErrorCode *status) { 
-  UPlugData *plug = NULL;
-  if(U_FAILURE(*status)) { return NULL; }
+  UPlugData *plug = nullptr;
+  if(U_FAILURE(*status)) { return nullptr; }
   plug = uplug_initPlugFromLibrary(libName, sym, config, status);
   uplug_loadPlug(plug, status);
 
@@ -632,7 +632,7 @@ U_CAPI UPlugLevel U_EXPORT2 uplug_getCurrentLevel() {
   return gCurrentLevel;
 }
 
-static UBool U_CALLCONV uplug_cleanup(void)
+static UBool U_CALLCONV uplug_cleanup()
 {
   int32_t i;
     
@@ -712,7 +712,7 @@ uplug_getPluginFile() {
 #if U_ENABLE_DYLOAD && !UCONFIG_NO_FILE_IO
   return plugin_file;
 #else
-  return NULL;
+  return nullptr;
 #endif
 }
 
@@ -728,7 +728,7 @@ uplug_init(UErrorCode *status) {
   const char *env = getenv("ICU_PLUGINS");
 
   if(U_FAILURE(*status)) return;
-  if(env != NULL) {
+  if(env != nullptr) {
     plugin_dir.append(env, -1, *status);
   }
   if(U_FAILURE(*status)) return;
@@ -791,7 +791,7 @@ uplug_init(UErrorCode *status) {
 #ifdef __MVS__
     if (iscics()) /* 12 Nov 2011 JAM */
     {
-        f = NULL;
+        f = nullptr;
     }
     else
 #endif
@@ -799,9 +799,9 @@ uplug_init(UErrorCode *status) {
         f = fopen(pluginFile.data(), "r");
     }
 
-    if(f != NULL) {
+    if(f != nullptr) {
       char linebuf[1024];
-      char *p, *libName=NULL, *symName=NULL, *config=NULL;
+      char *p, *libName=nullptr, *symName=nullptr, *config=nullptr;
       int32_t line = 0;
             
             
@@ -843,7 +843,7 @@ uplug_init(UErrorCode *status) {
           }
                     
           /* chop whitespace at the end of the config */
-          if(config!=NULL&&*config!=0) {
+          if(config!=nullptr&&*config!=0) {
             p = config+strlen(config);
             while(p>config&&isspace((int)*(--p))) {
               *p=0;
