@@ -1,12 +1,22 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 **********************************************************************
-*   Copyright (c) 2002, International Business Machines Corporation
+*   Copyright (c) 2002-2011, International Business Machines Corporation
 *   and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   02/04/2002  aliu        Creation.
 **********************************************************************
 */
+
+#ifndef FUNCREPL_H
+#define FUNCREPL_H
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/unifunct.h"
 #include "unicode/unirepl.h"
 
@@ -19,6 +29,7 @@ class Transliterator;
  * The input text to the transliterator is the output of another
  * UnicodeReplacer object.  That is, this replacer wraps another
  * replacer with a transliterator.
+ *
  * @author Alan Liu
  */
 class FunctionReplacer : public UnicodeFunctor, public UnicodeReplacer {
@@ -59,13 +70,13 @@ class FunctionReplacer : public UnicodeFunctor, public UnicodeReplacer {
     /**
      * Implement UnicodeFunctor
      */
-    virtual UnicodeFunctor* clone() const;
+    virtual FunctionReplacer* clone() const override;
 
     /**
      * UnicodeFunctor API.  Cast 'this' to a UnicodeReplacer* pointer
      * and return the pointer.
      */
-    virtual UnicodeReplacer* toReplacer() const;
+    virtual UnicodeReplacer* toReplacer() const override;
 
     /**
      * UnicodeReplacer API
@@ -73,20 +84,38 @@ class FunctionReplacer : public UnicodeFunctor, public UnicodeReplacer {
     virtual int32_t replace(Replaceable& text,
                             int32_t start,
                             int32_t limit,
-                            int32_t& cursor);
+                            int32_t& cursor) override;
 
     /**
      * UnicodeReplacer API
      */
     virtual UnicodeString& toReplacerPattern(UnicodeString& rule,
-                                             UBool escapeUnprintable) const;
+                                             UBool escapeUnprintable) const override;
+
+    /**
+     * Implement UnicodeReplacer
+     */
+    virtual void addReplacementSetTo(UnicodeSet& toUnionTo) const override;
 
     /**
      * UnicodeFunctor API
      */
-    virtual void setData(const TransliterationRuleData*);
+    virtual void setData(const TransliterationRuleData*) override;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     */
+    virtual UClassID getDynamicClassID() const override;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+#endif
+
 //eof
