@@ -1,7 +1,7 @@
 /*
  ******************************************************************************
  * © 2016 and later: Unicode, Inc. and others.                    *
- * License & terms of use: http://www.unicode.org/copyright.html#License      *
+ * License & terms of use: http://www.unicode.org/copyright.html      *
  ******************************************************************************
  ******************************************************************************
  * Copyright (C) 1998-2005, International Business Machines Corporation and   *
@@ -27,12 +27,12 @@
  * Read the text from a file. The text must start with a Unicode Byte
  * Order Mark (BOM) so that we know what order to read the bytes in.
  */
-const UChar *UnicodeReader::readFile(const char *fileName, GUISupport *guiSupport, int32_t &charCount)
+const char16_t *UnicodeReader::readFile(const char *fileName, GUISupport *guiSupport, int32_t &charCount)
 {
     FILE *f;
     int32_t fileSize;
     
-    UChar *charBuffer;
+    char16_t *charBuffer;
     char *byteBuffer;
     char startBytes[4] = {'\xA5', '\xA5', '\xA5', '\xA5'};
     char errorMessage[128];
@@ -41,7 +41,7 @@ const UChar *UnicodeReader::readFile(const char *fileName, GUISupport *guiSuppor
     
     f = fopen(fileName, "rb");
     
-    if( f == NULL ) {
+    if( f == nullptr ) {
         sprintf(errorMessage,"Couldn't open %s: %s \n", fileName, strerror(errno));
         guiSupport->postErrorMessage(errorMessage, "Text File Error");
         return 0;
@@ -108,7 +108,7 @@ const UChar *UnicodeReader::readFile(const char *fileName, GUISupport *guiSuppor
     delete[] byteBuffer;
     
     charCount = myText.length();
-    charBuffer = LE_NEW_ARRAY(UChar, charCount + 1);
+    charBuffer = LE_NEW_ARRAY(char16_t, charCount + 1);
     if(charBuffer == 0) {
         sprintf(errorMessage,"Couldn't get memory for reading %s: %s \n", fileName, strerror(errno));
         guiSupport->postErrorMessage(errorMessage, "Text File Error");
@@ -116,7 +116,7 @@ const UChar *UnicodeReader::readFile(const char *fileName, GUISupport *guiSuppor
     }
     
     myText.extract(0, myText.length(), charBuffer);
-    charBuffer[charCount] = 0;    // NULL terminate for easier reading in the debugger
+    charBuffer[charCount] = 0;    // NUL terminate for easier reading in the debugger
     
     return charBuffer;
 }
