@@ -15,7 +15,7 @@
 
 U_CDECL_BEGIN
 
-const UChar *uc_readFile(const char *fileName, gs_guiSupport *guiSupport, int32_t *charCount);
+const char16_t *uc_readFile(const char *fileName, gs_guiSupport *guiSupport, int32_t *charCount);
 
 U_CDECL_END
 
