@@ -30,7 +30,7 @@
 
 // Very simple example code - sticks a sortkey in the buffer
 // Not much error checking
-int32_t getSortKey_current(const char *locale, const UChar *string, int32_t sLen, uint8_t *buffer, int32_t bLen) {
+int32_t getSortKey_current(const char *locale, const char16_t *string, int32_t sLen, uint8_t *buffer, int32_t bLen) {
   UErrorCode status = U_ZERO_ERROR;
   UCollator *coll = ucol_open(locale, &status);
   if(U_FAILURE(status)) {
@@ -43,11 +43,11 @@ int32_t getSortKey_current(const char *locale, const UChar *string, int32_t sLen
 
 // This one can be used for passing to qsort function
 // Not thread safe or anything
-static UCollator *compareCollator = NULL;
+static UCollator *compareCollator = nullptr;
 
 int compare_current(const void *string1, const void *string2) {
-  if(compareCollator != NULL) {
-    UCollationResult res = ucol_strcoll(compareCollator, (UChar *) string1, -1, (UChar *) string2, -1);
+  if(compareCollator != nullptr) {
+    UCollationResult res = ucol_strcoll(compareCollator, (char16_t *) string1, -1, (char16_t *) string2, -1);
     if(res == UCOL_LESS) {
       return -1;
     } else if(res == UCOL_GREATER) {
@@ -65,13 +65,13 @@ void initCollator_current(const char *locale) {
   compareCollator = ucol_open(locale, &status);
 }
 
-void closeCollator_current(void) {
+void closeCollator_current() {
   ucol_close(compareCollator);
-  compareCollator = NULL;
+  compareCollator = nullptr;
 }
 
 
-extern "C" void test_current(UChar data[][5], uint32_t size, uint32_t maxlen, uint8_t keys[][32]) {
+extern "C" void test_current(char16_t data[][5], uint32_t size, uint32_t maxlen, uint8_t keys[][32]) {
   uint32_t i = 0;
   int32_t keySize = 0;
   UVersionInfo uvi;
@@ -87,7 +87,7 @@ extern "C" void test_current(UChar data[][5], uint32_t size, uint32_t maxlen, ui
   fprintf(stderr, "Done sortkeys, doing qsort test\n");
   
   initCollator_current("ja");
-  qsort(data, size, maxlen*sizeof(UChar), compare_current);
+  qsort(data, size, maxlen*sizeof(char16_t), compare_current);
   closeCollator_current();
 
   fprintf(stderr, "Done current!\n");
