@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2001, International Business Machines Corporation and
@@ -21,6 +23,9 @@
 #ifndef _CESCOLLTST
 #define _CESCOLLTST
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
 
 #include "cintltst.h"
 
@@ -35,6 +40,7 @@
  static    void TestTertiary(void);
 
 
+#endif /* #if !UCONFIG_NO_COLLATION */
     
 
 #endif
