@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2013, International Business Machines Corporation and
@@ -15,6 +17,10 @@
 */
 #ifndef _CRESTSTN
 #define _CRESTSTN
+
+#include "unicode/utypes.h"
+#include "unicode/ures.h"
+
 /* C TEST FOR NEW RESOURCEBUNDLE API*/
 #include "cintltst.h"
 
@@ -75,8 +81,6 @@ static void TestJB3763(void);
 
 static void TestXPath(void);
 
-static void TestStackReuse(void);
-
 /**
 * extensive subtests called by TestResourceBundles
 **/
