@@ -17,6 +17,7 @@
 #include "cintltst.h"
 #include "cmemory.h"
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <time.h>
 
@@ -44,15 +45,15 @@ static uint64_t randomInt64(void)
 {
     int64_t ran = 0;
     int32_t i;
-    static UBool initialized = FALSE;
+    static UBool initialized = false;
 
     if (!initialized) {
         srand((unsigned)time(NULL));
-        initialized = TRUE;
+        initialized = true;
     }
 
     /* Assume rand has at least 12 bits of precision */
-    for (i = 0; i < sizeof(ran); i += 1) {
+    for (i = 0; i < (int32_t)sizeof(ran); i += 1) {
         ((char*)&ran)[i] = (char)((rand() & 0x0FF0) >> 4);
     }
 
