@@ -26,19 +26,19 @@ public:
 public:
     AstroTest();
 
-    void TestSolarLongitude(void);
+    void TestSolarLongitude();
 
-    void TestLunarPosition(void);
+    void TestLunarPosition();
 
-    void TestCoordinates(void);
+    void TestCoordinates();
 
-    void TestCoverage(void);
+    void TestCoverage();
 
-    void TestSunriseTimes(void);
+    void TestSunriseTimes();
 
-    void TestBasics(void);
+    void TestBasics();
     
-    void TestMoonAge(void);
+    void TestMoonAge();
  private:
     void initAstro(UErrorCode&);
     void closeAstro(UErrorCode&);
