@@ -56,7 +56,7 @@ enum {
 ConversionTest::ConversionTest() {
     UErrorCode errorCode=U_ZERO_ERROR;
     utf8Cnv=ucnv_open("UTF-8", &errorCode);
-    ucnv_setToUCallBack(utf8Cnv, UCNV_TO_U_CALLBACK_STOP, NULL, NULL, NULL, &errorCode);
+    ucnv_setToUCallBack(utf8Cnv, UCNV_TO_U_CALLBACK_STOP, nullptr, nullptr, nullptr, &errorCode);
     if(U_FAILURE(errorCode)) {
         errln("unable to open UTF-8 converter");
     }
@@ -126,7 +126,7 @@ ConversionTest::TestToUnicode() {
                 offsetsLength=0;
                 cc.offsets=testCase->getIntVector(offsetsLength, "offsets", errorCode);
                 if(offsetsLength==0) {
-                    cc.offsets=NULL;
+                    cc.offsets=nullptr;
                 } else if(offsetsLength!=unicode.length()) {
                     errln("toUnicode[%d] unicode[%d] and offsets[%d] must have the same length",
                             i, unicode.length(), offsetsLength);
@@ -168,12 +168,12 @@ ConversionTest::TestToUnicode() {
                     callback=UCNV_TO_U_CALLBACK_ESCAPE;
                     break;
                 default:
-                    callback=NULL;
+                    callback=nullptr;
                     break;
                 }
-                option=callback==NULL ? cbopt : cbopt+1;
+                option=callback==nullptr ? cbopt : cbopt+1;
                 if(*option==0) {
-                    option=NULL;
+                    option=nullptr;
                 }
 
                 cc.invalidChars=testCase->getBinary(cc.invalidLength, "invalidChars", errorCode);
@@ -208,7 +208,7 @@ ConversionTest::TestFromUnicode() {
     TestDataModule *dataModule;
     TestData *testData;
     const DataMap *testCase;
-    const UChar *p;
+    const char16_t *p;
     UErrorCode errorCode;
     int32_t i, length;
 
@@ -239,7 +239,7 @@ ConversionTest::TestFromUnicode() {
                 offsetsLength=0;
                 cc.offsets=testCase->getIntVector(offsetsLength, "offsets", errorCode);
                 if(offsetsLength==0) {
-                    cc.offsets=NULL;
+                    cc.offsets=nullptr;
                 } else if(offsetsLength!=cc.bytesLength) {
                     errln("fromUnicode[%d] bytes[%d] and offsets[%d] must have the same length",
                             i, cc.bytesLength, offsetsLength);
@@ -263,7 +263,7 @@ ConversionTest::TestFromUnicode() {
                 s=testCase->getString("callback", errorCode);
                 cc.setSub=0; // default: no subchar
 
-                if((index=s.indexOf((UChar)0))>0) {
+                if((index=s.indexOf((char16_t)0))>0) {
                     // read NUL-separated subchar first, if any
                     // copy the subchar from Latin-1 characters
                     // start after the NUL
@@ -286,7 +286,7 @@ ConversionTest::TestFromUnicode() {
 
                     // remove the NUL and subchar from s
                     s.truncate(index);
-                } else if((index=s.indexOf((UChar)0x3d))>0) /* '=' */ {
+                } else if((index=s.indexOf((char16_t)0x3d))>0) /* '=' */ {
                     // read a substitution string, separated by an equal sign
                     p=s.getBuffer()+index+1;
                     length=s.length()-(index+1);
@@ -319,12 +319,12 @@ ConversionTest::TestFromUnicode() {
                     callback=UCNV_FROM_U_CALLBACK_ESCAPE;
                     break;
                 default:
-                    callback=NULL;
+                    callback=nullptr;
                     break;
                 }
-                option=callback==NULL ? cbopt : cbopt+1;
+                option=callback==nullptr ? cbopt : cbopt+1;
                 if(*option==0) {
-                    option=NULL;
+                    option=nullptr;
                 }
 
                 invalidUChars=testCase->getString("invalidUChars", errorCode);
@@ -349,7 +349,7 @@ ConversionTest::TestFromUnicode() {
     }
 }
 
-static const UChar ellipsis[]={ 0x2e, 0x2e, 0x2e };
+static const char16_t ellipsis[]={ 0x2e, 0x2e, 0x2e };
 
 void
 ConversionTest::TestGetUnicodeSet() {
@@ -401,7 +401,7 @@ ConversionTest::TestGetUnicodeSet() {
                 mapnotSet.clear();
 
                 pos.setIndex(0);
-                mapSet.applyPattern(map, pos, 0, NULL, errorCode);
+                mapSet.applyPattern(map, pos, 0, nullptr, errorCode);
                 if(U_FAILURE(errorCode) || pos.getIndex()!=map.length()) {
                     errln("error creating the map set for conversion/getUnicodeSet test case %d - %s\n"
                           "    error index %d  index %d  U+%04x",
@@ -411,7 +411,7 @@ ConversionTest::TestGetUnicodeSet() {
                 }
 
                 pos.setIndex(0);
-                mapnotSet.applyPattern(mapnot, pos, 0, NULL, errorCode);
+                mapnotSet.applyPattern(mapnot, pos, 0, nullptr, errorCode);
                 if(U_FAILURE(errorCode) || pos.getIndex()!=mapnot.length()) {
                     errln("error creating the mapnot set for conversion/getUnicodeSet test case %d - %s\n"
                           "    error index %d  index %d  U+%04x",
@@ -442,7 +442,7 @@ ConversionTest::TestGetUnicodeSet() {
                 // are there items that must be in cnvSet but are not?
                 (diffSet=mapSet).removeAll(cnvSet);
                 if(!diffSet.isEmpty()) {
-                    diffSet.toPattern(s, TRUE);
+                    diffSet.toPattern(s, true);
                     if(s.length()>100) {
                         s.replace(100, 0x7fffffff, ellipsis, UPRV_LENGTHOF(ellipsis));
                     }
@@ -454,7 +454,7 @@ ConversionTest::TestGetUnicodeSet() {
                 // are there items that must not be in cnvSet but are?
                 (diffSet=mapnotSet).retainAll(cnvSet);
                 if(!diffSet.isEmpty()) {
-                    diffSet.toPattern(s, TRUE);
+                    diffSet.toPattern(s, true);
                     if(s.length()>100) {
                         s.replace(100, 0x7fffffff, ellipsis, UPRV_LENGTHOF(ellipsis));
                     }
@@ -476,13 +476,13 @@ U_CDECL_BEGIN
 static void U_CALLCONV
 getUnicodeSetCallback(const void *context,
                       UConverterFromUnicodeArgs * /*fromUArgs*/,
-                      const UChar* /*codeUnits*/,
+                      const char16_t* /*codeUnits*/,
                       int32_t /*length*/,
                       UChar32 codePoint,
                       UConverterCallbackReason reason,
                       UErrorCode *pErrorCode) {
     if(reason<=UCNV_IRREGULAR) {
-        ((UnicodeSet *)context)->remove(codePoint);  // the converter cannot convert this code point
+        static_cast<UnicodeSet *>(const_cast<void*>(context))->remove(codePoint);  // the converter cannot convert this code point
         *pErrorCode=U_ZERO_ERROR;                    // skip
     }  // else ignore the reset, close and clone calls.
 }
@@ -500,35 +500,35 @@ ConversionTest::TestGetUnicodeSet2() {
         cpLimit=0x110000;
         s0Length=0x10000+0x200000;  // BMP + surrogate pairs
     }
-    UChar *s0=new UChar[s0Length];
-    if(s0==NULL) {
+    char16_t *s0=new char16_t[s0Length];
+    if(s0==nullptr) {
         return;
     }
-    UChar *s=s0;
+    char16_t *s=s0;
     UChar32 c;
-    UChar c2;
+    char16_t c2;
     // low BMP
     for(c=0; c<=0xd7ff; ++c) {
-        *s++=(UChar)c;
+        *s++=(char16_t)c;
     }
     // trail surrogates
     for(c=0xdc00; c<=0xdfff; ++c) {
-        *s++=(UChar)c;
+        *s++=(char16_t)c;
     }
     // lead surrogates
     // (after trails so that there is not even one surrogate pair in between)
     for(c=0xd800; c<=0xdbff; ++c) {
-        *s++=(UChar)c;
+        *s++=(char16_t)c;
     }
     // high BMP
     for(c=0xe000; c<=0xffff; ++c) {
-        *s++=(UChar)c;
+        *s++=(char16_t)c;
     }
     // supplementary code points = surrogate pairs
     if(cpLimit==0x110000) {
         for(c=0xd800; c<=0xdbff; ++c) {
             for(c2=0xdc00; c2<=0xdfff; ++c2) {
-                *s++=(UChar)c;
+                *s++=(char16_t)c;
                 *s++=c2;
             }
         }
@@ -562,7 +562,7 @@ ConversionTest::TestGetUnicodeSet2() {
             continue;
         }
         UnicodeSet expected;
-        ucnv_setFromUCallBack(cnv.getAlias(), getUnicodeSetCallback, &expected, NULL, NULL, &errorCode);
+        ucnv_setFromUCallBack(cnv.getAlias(), getUnicodeSetCallback, &expected, nullptr, nullptr, &errorCode);
         if(U_FAILURE(errorCode)) {
             errln("failed to set the callback on converter %s - %s", cnvNames[i], u_errorName(errorCode));
             continue;
@@ -570,7 +570,7 @@ ConversionTest::TestGetUnicodeSet2() {
         UConverterUnicodeSet which;
         for(which=UCNV_ROUNDTRIP_SET; which<UCNV_SET_COUNT; which=(UConverterUnicodeSet)((int)which+1)) {
             if(which==UCNV_ROUNDTRIP_AND_FALLBACK_SET) {
-                ucnv_setFallback(cnv.getAlias(), TRUE);
+                ucnv_setFallback(cnv.getAlias(), true);
             }
             expected.add(0, cpLimit-1);
             s=s0;
@@ -578,7 +578,7 @@ ConversionTest::TestGetUnicodeSet2() {
             do {
                 char *t=buffer;
                 flush=(UBool)(s==s0+s0Length);
-                ucnv_fromUnicode(cnv.getAlias(), &t, buffer+sizeof(buffer), (const UChar **)&s, s0+s0Length, NULL, flush, &errorCode);
+                ucnv_fromUnicode(cnv.getAlias(), &t, buffer+sizeof(buffer), (const char16_t **)&s, s0+s0Length, nullptr, flush, &errorCode);
                 if(U_FAILURE(errorCode)) {
                     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
                         errorCode=U_ZERO_ERROR;
@@ -617,7 +617,7 @@ ConversionTest::TestGetUnicodeSet2() {
                 // are there items that must be in the set but are not?
                 (diffSet=expected).removeAll(set);
                 if(!diffSet.isEmpty()) {
-                    diffSet.toPattern(out, TRUE);
+                    diffSet.toPattern(out, true);
                     if(out.length()>100) {
                         out.replace(100, 0x7fffffff, ellipsis, UPRV_LENGTHOF(ellipsis));
                     }
@@ -629,7 +629,7 @@ ConversionTest::TestGetUnicodeSet2() {
                 // are there items that must not be in the set but are?
                 (diffSet=set).removeAll(expected);
                 if(!diffSet.isEmpty()) {
-                    diffSet.toPattern(out, TRUE);
+                    diffSet.toPattern(out, true);
                     if(out.length()>100) {
                         out.replace(100, 0x7fffffff, ellipsis, UPRV_LENGTHOF(ellipsis));
                     }
@@ -679,7 +679,7 @@ ConversionTest::TestDefaultIgnorableCallback() {
     }
 
     // set callback for the converter 
-    ucnv_setFromUCallBack(cnv.getAlias(), UCNV_FROM_U_CALLBACK_SUBSTITUTE, NULL, NULL, NULL, &status);
+    ucnv_setFromUCallBack(cnv.getAlias(), UCNV_FROM_U_CALLBACK_SUBSTITUTE, nullptr, nullptr, nullptr, &status);
 
     UChar32 input[1];
     char output[10];
@@ -729,10 +729,10 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     char result[20];
     char *target = result;
     const char *targetLimit = result + sizeof(result);
-    UChar buffer16[20];
-    UChar *pivotSource = buffer16;
-    UChar *pivotTarget = buffer16;
-    const UChar *pivotLimit = buffer16 + UPRV_LENGTHOF(buffer16);
+    char16_t buffer16[20];
+    char16_t *pivotSource = buffer16;
+    char16_t *pivotTarget = buffer16;
+    const char16_t *pivotLimit = buffer16 + UPRV_LENGTHOF(buffer16);
     int32_t length;
 
     // Convert with insufficient target capacity.
@@ -740,7 +740,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, result + 2, &source, sourceLimit,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, FALSE, errorCode);
+                   false, false, errorCode);
     assertEquals("overflow", U_BUFFER_OVERFLOW_ERROR, errorCode.reset());
     length = (int32_t)(target - result);
     assertEquals("number of bytes written", 2, length);
@@ -750,7 +750,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, targetLimit, &source, sourceLimit,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, TRUE, errorCode);
+                   false, true, errorCode);
 
     assertSuccess("UTF-8->UTF-8", errorCode);
     length = (int32_t)(target - result);
@@ -773,7 +773,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, result + 3, &source, sourceLimit,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, FALSE, errorCode);
+                   false, false, errorCode);
     assertEquals("text2 overflow", U_BUFFER_OVERFLOW_ERROR, errorCode.reset());
     length = (int32_t)(target - result);
     assertEquals("text2 number of bytes written", 3, length);
@@ -783,7 +783,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, targetLimit, &source, sourceLimit,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, TRUE, errorCode);
+                   false, true, errorCode);
 
     assertSuccess("text2 UTF-8->UTF-8", errorCode);
     length = (int32_t)(target - result);
@@ -810,7 +810,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, targetLimit, &source, source + 2,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, TRUE, errorCode);
+                   false, true, errorCode);
     assertEquals("illFormed truncated", U_TRUNCATED_CHAR_FOUND, errorCode.reset());
     length = (int32_t)(target - result);
     assertEquals("illFormed number of bytes written", 0, length);
@@ -826,7 +826,7 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
                    &target, targetLimit, &source, sourceLimit,
                    buffer16, &pivotSource, &pivotTarget, pivotLimit,
-                   FALSE, TRUE, errorCode);
+                   false, true, errorCode);
 
     assertEquals("illFormed trail byte", U_ILLEGAL_CHAR_FOUND, errorCode.reset());
     length = (int32_t)(target - result);
@@ -866,16 +866,16 @@ ConversionTest::TestUTF8ToUTF8Streaming() {
     char* target = result;
     const char* targetLimit = result + targetLen;
 
-    UChar buffer16[20];
-    UChar* pivotSource = buffer16;
-    UChar* pivotTarget = buffer16;
-    const UChar* pivotLimit = buffer16 + UPRV_LENGTHOF(buffer16);
+    char16_t buffer16[20];
+    char16_t* pivotSource = buffer16;
+    char16_t* pivotTarget = buffer16;
+    const char16_t* pivotLimit = buffer16 + UPRV_LENGTHOF(buffer16);
 
     int32_t length;
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
         &target, result + targetLen, &source, sourceLimit,
         buffer16, &pivotSource, &pivotTarget, pivotLimit,
-        FALSE, FALSE, errorCode);
+        false, false, errorCode);
 
     length = (int32_t)(target - result);
     targetLen -= length;
@@ -888,7 +888,7 @@ ConversionTest::TestUTF8ToUTF8Streaming() {
     ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
         &target, targetLimit, &source, sourceLimit,
         buffer16, &pivotSource, &pivotTarget, pivotLimit,
-        FALSE, TRUE, errorCode);
+        false, true, errorCode);
 
     length = (int32_t)(target - result - length);
     targetLen -= length;
@@ -902,11 +902,11 @@ ConversionTest::TestUTF8ToUTF8Streaming() {
 
 UConverter *
 ConversionTest::cnv_open(const char *name, UErrorCode &errorCode) {
-    if(name!=NULL && *name=='+') {
+    if(name!=nullptr && *name=='+') {
         // Converter names that start with '+' are ignored in ICU4J tests.
         ++name;
     }
-    if(name!=NULL && *name=='*') {
+    if(name!=nullptr && *name=='*') {
         /* loadTestData(): set the data directory */
         return ucnv_openPackage(loadTestData(errorCode), name+1, &errorCode);
     } else {
@@ -944,7 +944,7 @@ printBytes(const uint8_t *bytes, int32_t length, char *out) {
 }
 
 static char *
-printUnicode(const UChar *unicode, int32_t length, char *out) {
+printUnicode(const char16_t *unicode, int32_t length, char *out) {
     UChar32 c;
     int32_t i;
 
@@ -973,7 +973,7 @@ static char *
 printOffsets(const int32_t *offsets, int32_t length, char *out) {
     int32_t i, o, d;
 
-    if(offsets==NULL) {
+    if(offsets==nullptr) {
         length=0;
     }
 
@@ -1006,12 +1006,12 @@ printOffsets(const int32_t *offsets, int32_t length, char *out) {
 
 static int32_t
 stepToUnicode(ConversionCase &cc, UConverter *cnv,
-              UChar *result, int32_t resultCapacity,
+              char16_t *result, int32_t resultCapacity,
               int32_t *resultOffsets, /* also resultCapacity */
               int32_t step,
               UErrorCode *pErrorCode) {
     const char *source, *sourceLimit, *bytesLimit;
-    UChar *target, *targetLimit, *resultLimit;
+    char16_t *target, *targetLimit, *resultLimit;
     UBool flush;
 
     source=(const char *)cc.bytes;
@@ -1034,10 +1034,10 @@ stepToUnicode(ConversionCase &cc, UConverter *cnv,
             // start with empty partial buffers
             sourceLimit=source;
             targetLimit=target;
-            flush=FALSE;
+            flush=false;
 
             // output offsets only for bulk conversion
-            resultOffsets=NULL;
+            resultOffsets=nullptr;
         }
 
         for(;;) {
@@ -1126,7 +1126,7 @@ stepToUnicode(ConversionCase &cc, UConverter *cnv,
                     break;
                 }
                 if(c<=0xffff) {
-                    *target++=(UChar)c;
+                    *target++=(char16_t)c;
                 } else {
                     *target++=U16_LEAD(c);
                     if(target==resultLimit) {
@@ -1141,7 +1141,7 @@ stepToUnicode(ConversionCase &cc, UConverter *cnv,
                     ++step;
                 }
             } else /* step is even */ {
-                // allow only one UChar output
+                // allow only one char16_t output
                 targetLimit=target<resultLimit ? target+1 : resultLimit;
 
                 // as with ucnv_getNextUChar(), we always flush (if we go to bytesLimit)
@@ -1158,7 +1158,7 @@ stepToUnicode(ConversionCase &cc, UConverter *cnv,
                 ucnv_toUnicode(cnv,
                     &target, targetLimit,
                     &source, sourceLimit,
-                    NULL, (UBool)(sourceLimit==bytesLimit), pErrorCode);
+                    nullptr, (UBool)(sourceLimit==bytesLimit), pErrorCode);
 
                 // check pointers and errors
                 if(*pErrorCode==U_BUFFER_OVERFLOW_ERROR) {
@@ -1183,7 +1183,7 @@ stepToUnicode(ConversionCase &cc, UConverter *cnv,
                         break;
                     }
 
-                    // we are done (flush==TRUE) but we continue, to get the index out of bounds error above
+                    // we are done (flush==true) but we continue, to get the index out of bounds error above
                 }
 
                 --step;
@@ -1204,21 +1204,21 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         errcheckln(errorCode, "toUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_open() failed - %s",
                 cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, errorCode.errorName());
         errorCode.reset();
-        return FALSE;
+        return false;
     }
 
     // set the callback
-    if(callback!=NULL) {
-        ucnv_setToUCallBack(cnv.getAlias(), callback, option, NULL, NULL, errorCode);
+    if(callback!=nullptr) {
+        ucnv_setToUCallBack(cnv.getAlias(), callback, option, nullptr, nullptr, errorCode);
         if(U_FAILURE(errorCode)) {
             errln("toUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setToUCallBack() failed - %s",
                     cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
-            return FALSE;
+            return false;
         }
     }
 
     int32_t resultOffsets[256];
-    UChar result[256];
+    char16_t result[256];
     int32_t resultLength;
     UBool ok;
 
@@ -1240,7 +1240,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
     };
     int32_t i, step;
 
-    ok=TRUE;
+    ok=true;
     for(i=0; i<UPRV_LENGTHOF(steps) && ok; ++i) {
         step=steps[i].step;
         if(step<0 && !cc.finalFlush) {
@@ -1250,7 +1250,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         }
         if(step!=0) {
             // bulk test is first, then offsets are not checked any more
-            cc.offsets=NULL;
+            cc.offsets=nullptr;
         }
         else {
             for (int32_t i = 0; i < UPRV_LENGTHOF(resultOffsets); i++) {
@@ -1263,23 +1263,23 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         errorCode.reset();
         resultLength=stepToUnicode(cc, cnv.getAlias(),
                                 result, UPRV_LENGTHOF(result),
-                                step==0 ? resultOffsets : NULL,
+                                step==0 ? resultOffsets : nullptr,
                                 step, errorCode);
         ok=checkToUnicode(
                 cc, cnv.getAlias(), steps[i].name,
                 result, resultLength,
-                cc.offsets!=NULL ? resultOffsets : NULL,
+                cc.offsets!=nullptr ? resultOffsets : nullptr,
                 errorCode);
         if(errorCode.isFailure() || !cc.finalFlush) {
             // reset if an error occurred or we did not flush
             // otherwise do nothing to make sure that flushing resets
             ucnv_resetToUnicode(cnv.getAlias());
         }
-        if (cc.offsets != NULL && resultOffsets[resultLength] != -1) {
+        if (cc.offsets != nullptr && resultOffsets[resultLength] != -1) {
             errln("toUnicode[%d](%s) Conversion wrote too much to offsets at index %d",
                 cc.caseNr, cc.charset, resultLength);
         }
-        if (result[resultLength] != (UChar)-1) {
+        if (result[resultLength] != (char16_t)-1) {
             errln("toUnicode[%d](%s) Conversion wrote too much to result at index %d",
                 cc.caseNr, cc.charset, resultLength);
         }
@@ -1298,7 +1298,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         ok=checkToUnicode(
                 cc, cnv.getAlias(), "toUChars",
                 result, resultLength,
-                NULL,
+                nullptr,
                 errorCode);
         if(!ok) {
             break;
@@ -1308,7 +1308,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         // keep the correct result for simple checking
         errorCode.reset();
         resultLength=ucnv_toUChars(cnv.getAlias(),
-                        NULL, 0,
+                        nullptr, 0,
                         (const char *)cc.bytes, cc.bytesLength,
                         errorCode);
         if(errorCode.get()==U_STRING_NOT_TERMINATED_WARNING || errorCode.get()==U_BUFFER_OVERFLOW_ERROR) {
@@ -1317,7 +1317,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
         ok=checkToUnicode(
                 cc, cnv.getAlias(), "preflight toUChars",
                 result, resultLength,
-                NULL,
+                nullptr,
                 errorCode);
         break;
     }
@@ -1328,7 +1328,7 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
 
 UBool
 ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *name,
-                               const UChar *result, int32_t resultLength,
+                               const char16_t *result, int32_t resultLength,
                                const int32_t *resultOffsets,
                                UErrorCode resultErrorCode) {
     char resultInvalidChars[8];
@@ -1337,8 +1337,8 @@ ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *
 
     const char *msg;
 
-    // reset the message; NULL will mean "ok"
-    msg=NULL;
+    // reset the message; nullptr will mean "ok"
+    msg=nullptr;
 
     errorCode=U_ZERO_ERROR;
     resultInvalidLength=sizeof(resultInvalidChars);
@@ -1346,7 +1346,7 @@ ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *
     if(U_FAILURE(errorCode)) {
         errln("toUnicode[%d](%s cb=\"%s\" fb=%d flush=%d %s) ucnv_getInvalidChars() failed - %s",
                 cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, name, u_errorName(errorCode));
-        return FALSE;
+        return false;
     }
 
     // check everything that might have gone wrong
@@ -1354,7 +1354,7 @@ ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *
         msg="wrong result length";
     } else if(0!=u_memcmp(cc.unicode, result, cc.unicodeLength)) {
         msg="wrong result string";
-    } else if(cc.offsets!=NULL && 0!=memcmp(cc.offsets, resultOffsets, cc.unicodeLength*sizeof(*cc.offsets))) {
+    } else if(cc.offsets!=nullptr && 0!=memcmp(cc.offsets, resultOffsets, cc.unicodeLength*sizeof(*cc.offsets))) {
         msg="wrong offsets";
     } else if(cc.outErrorCode!=resultErrorCode) {
         msg="wrong error code";
@@ -1364,8 +1364,8 @@ ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *
         msg="wrong last invalid input";
     }
 
-    if(msg==NULL) {
-        return TRUE;
+    if(msg==nullptr) {
+        return true;
     } else {
         char buffer[2000]; // one buffer for all strings
         char *s, *bytesString, *unicodeString, *resultString,
@@ -1404,7 +1404,7 @@ ConversionTest::checkToUnicode(ConversionCase &cc, UConverter *cnv, const char *
               u_errorName(cc.outErrorCode), u_errorName(resultErrorCode),
               invalidCharsString, resultInvalidCharsString);
 
-        return FALSE;
+        return false;
     }
 }
 
@@ -1417,8 +1417,8 @@ stepFromUTF8(ConversionCase &cc,
              int32_t step,
              UErrorCode *pErrorCode) {
     const char *source, *sourceLimit, *utf8Limit;
-    UChar pivotBuffer[32];
-    UChar *pivotSource, *pivotTarget, *pivotLimit;
+    char16_t pivotBuffer[32];
+    char16_t *pivotSource, *pivotTarget, *pivotLimit;
     char *target, *targetLimit, *resultLimit;
     UBool flush;
 
@@ -1444,7 +1444,7 @@ stepFromUTF8(ConversionCase &cc,
         // start with empty partial buffers
         sourceLimit=source;
         targetLimit=target;
-        flush=FALSE;
+        flush=false;
 
         // empty pivot is not allowed, make it of length step
         pivotLimit=pivotBuffer+step;
@@ -1460,7 +1460,7 @@ stepFromUTF8(ConversionCase &cc,
             &target, targetLimit,
             &source, sourceLimit,
             pivotBuffer, &pivotSource, &pivotTarget, pivotLimit,
-            FALSE, flush, pErrorCode);
+            false, flush, pErrorCode);
 
         // check pointers and errors
         if(source>sourceLimit || target>targetLimit) {
@@ -1522,7 +1522,7 @@ stepFromUnicode(ConversionCase &cc, UConverter *cnv,
                 int32_t *resultOffsets, /* also resultCapacity */
                 int32_t step,
                 UErrorCode *pErrorCode) {
-    const UChar *source, *sourceLimit, *unicodeLimit;
+    const char16_t *source, *sourceLimit, *unicodeLimit;
     char *target, *targetLimit, *resultLimit;
     UBool flush;
 
@@ -1545,10 +1545,10 @@ stepFromUnicode(ConversionCase &cc, UConverter *cnv,
         // start with empty partial buffers
         sourceLimit=source;
         targetLimit=target;
-        flush=FALSE;
+        flush=false;
 
         // output offsets only for bulk conversion
-        resultOffsets=NULL;
+        resultOffsets=nullptr;
     }
 
     for(;;) {
@@ -1614,18 +1614,18 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
     if(U_FAILURE(errorCode)) {
         errcheckln(errorCode, "fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_open() failed - %s",
                 cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
-        return FALSE;
+        return false;
     }
     ucnv_resetToUnicode(utf8Cnv);
 
     // set the callback
-    if(callback!=NULL) {
-        ucnv_setFromUCallBack(cnv, callback, option, NULL, NULL, &errorCode);
+    if(callback!=nullptr) {
+        ucnv_setFromUCallBack(cnv, callback, option, nullptr, nullptr, &errorCode);
         if(U_FAILURE(errorCode)) {
             errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setFromUCallBack() failed - %s",
                     cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
             ucnv_close(cnv);
-            return FALSE;
+            return false;
         }
     }
 
@@ -1643,7 +1643,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
             errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setSubstChars() failed - %s",
                     cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
             ucnv_close(cnv);
-            return FALSE;
+            return false;
         }
     } else if(cc.setSub<0) {
         ucnv_setSubstString(cnv, cc.subString, -1, &errorCode);
@@ -1651,7 +1651,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
             errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setSubstString() failed - %s",
                     cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
             ucnv_close(cnv);
-            return FALSE;
+            return false;
         }
     }
 
@@ -1684,7 +1684,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
     };
     int32_t i, step;
 
-    ok=TRUE;
+    ok=true;
     for(i=0; i<UPRV_LENGTHOF(steps) && ok; ++i) {
         step=steps[i].step;
         for (int32_t i = 0; i < UPRV_LENGTHOF(resultOffsets); i++) {
@@ -1696,12 +1696,12 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
         errorCode=U_ZERO_ERROR;
         resultLength=stepFromUnicode(cc, cnv,
                                 result, UPRV_LENGTHOF(result),
-                                step==0 ? resultOffsets : NULL,
+                                step==0 ? resultOffsets : nullptr,
                                 step, &errorCode);
         ok=checkFromUnicode(
                 cc, cnv, steps[i].name,
                 (uint8_t *)result, resultLength,
-                cc.offsets!=NULL ? resultOffsets : NULL,
+                cc.offsets!=nullptr ? resultOffsets : nullptr,
                 errorCode);
         if(U_FAILURE(errorCode) || !cc.finalFlush) {
             // reset if an error occurred or we did not flush
@@ -1718,7 +1718,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
         }
 
         // bulk test is first, then offsets are not checked any more
-        cc.offsets=NULL;
+        cc.offsets=nullptr;
 
         // test direct conversion from UTF-8
         if(cc.utf8Length>=0) {
@@ -1729,7 +1729,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
             ok=checkFromUnicode(
                     cc, cnv, steps[i].utf8Name,
                     (uint8_t *)result, resultLength,
-                    NULL,
+                    nullptr,
                     errorCode);
             if(U_FAILURE(errorCode) || !cc.finalFlush) {
                 // reset if an error occurred or we did not flush
@@ -1753,7 +1753,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
         ok=checkFromUnicode(
                 cc, cnv, "fromUChars",
                 (uint8_t *)result, resultLength,
-                NULL,
+                nullptr,
                 errorCode);
         if(!ok) {
             break;
@@ -1763,7 +1763,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
         // keep the correct result for simple checking
         errorCode=U_ZERO_ERROR;
         resultLength=ucnv_fromUChars(cnv,
-                        NULL, 0,
+                        nullptr, 0,
                         cc.unicode, cc.unicodeLength,
                         &errorCode);
         if(errorCode==U_STRING_NOT_TERMINATED_WARNING || errorCode==U_BUFFER_OVERFLOW_ERROR) {
@@ -1772,7 +1772,7 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
         ok=checkFromUnicode(
                 cc, cnv, "preflight fromUChars",
                 (uint8_t *)result, resultLength,
-                NULL,
+                nullptr,
                 errorCode);
         break;
     }
@@ -1786,14 +1786,14 @@ ConversionTest::checkFromUnicode(ConversionCase &cc, UConverter *cnv, const char
                                  const uint8_t *result, int32_t resultLength,
                                  const int32_t *resultOffsets,
                                  UErrorCode resultErrorCode) {
-    UChar resultInvalidUChars[8];
+    char16_t resultInvalidUChars[8];
     int8_t resultInvalidLength;
     UErrorCode errorCode;
 
     const char *msg;
 
-    // reset the message; NULL will mean "ok"
-    msg=NULL;
+    // reset the message; nullptr will mean "ok"
+    msg=nullptr;
 
     errorCode=U_ZERO_ERROR;
     resultInvalidLength=UPRV_LENGTHOF(resultInvalidUChars);
@@ -1801,7 +1801,7 @@ ConversionTest::checkFromUnicode(ConversionCase &cc, UConverter *cnv, const char
     if(U_FAILURE(errorCode)) {
         errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d %s) ucnv_getInvalidUChars() failed - %s",
                 cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, name, u_errorName(errorCode));
-        return FALSE;
+        return false;
     }
 
     // check everything that might have gone wrong
@@ -1809,7 +1809,7 @@ ConversionTest::checkFromUnicode(ConversionCase &cc, UConverter *cnv, const char
         msg="wrong result length";
     } else if(0!=memcmp(cc.bytes, result, cc.bytesLength)) {
         msg="wrong result string";
-    } else if(cc.offsets!=NULL && 0!=memcmp(cc.offsets, resultOffsets, cc.bytesLength*sizeof(*cc.offsets))) {
+    } else if(cc.offsets!=nullptr && 0!=memcmp(cc.offsets, resultOffsets, cc.bytesLength*sizeof(*cc.offsets))) {
         msg="wrong offsets";
     } else if(cc.outErrorCode!=resultErrorCode) {
         msg="wrong error code";
@@ -1819,8 +1819,8 @@ ConversionTest::checkFromUnicode(ConversionCase &cc, UConverter *cnv, const char
         msg="wrong last invalid input";
     }
 
-    if(msg==NULL) {
-        return TRUE;
+    if(msg==nullptr) {
+        return true;
     } else {
         char buffer[2000]; // one buffer for all strings
         char *s, *unicodeString, *bytesString, *resultString,
@@ -1859,7 +1859,7 @@ ConversionTest::checkFromUnicode(ConversionCase &cc, UConverter *cnv, const char
               u_errorName(cc.outErrorCode), u_errorName(resultErrorCode),
               invalidCharsString, resultInvalidUCharsString);
 
-        return FALSE;
+        return false;
     }
 }
 
