@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2016, International Business Machines Corporation and
@@ -45,7 +47,7 @@ DateFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const char*
         CASE(14,Test4104136)
         CASE(15,Test4104522)
         CASE(16,Test4106807)
-        CASE(17,Test4108407) 
+        CASE(17,Test4108407)
         CASE(18,Test4134203)
         CASE(19,Test4151631)
         CASE(20,Test4151706)
@@ -57,10 +59,13 @@ DateFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const char*
         CASE(26,Test5554)
         CASE(27,Test9237)
         CASE(28,TestParsing)
-        CASE(29,TestT10334)
-        CASE(30,TestT10619)
-        CASE(31,TestT10855)
-        CASE(32,TestT10906)
+        CASE(29,Test12902_yWithGregoCalInThaiLoc)
+        CASE(30,TestT10334)
+        CASE(31,TestT10619)
+        CASE(32,TestT10855)
+        CASE(33,TestT10858)
+        CASE(34,TestT10906)
+        CASE(35,TestT13380)
         default: name = ""; break;
     }
 }
@@ -68,19 +73,19 @@ DateFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const char*
 /**
  * @bug 4029195
  */
-void DateFormatRegressionTest::Test4029195(void) 
+void DateFormatRegressionTest::Test4029195() 
 {
     UErrorCode status = U_ZERO_ERROR;
 
     UDate today = Calendar::getNow();
     logln((UnicodeString) "today: " + today);
 
-    SimpleDateFormat *sdf = (SimpleDateFormat*) DateFormat::createDateInstance();
+    SimpleDateFormat *sdf = dynamic_cast<SimpleDateFormat*>(DateFormat::createDateInstance());
     if (failure(status, "SimpleDateFormat::createDateInstance")) {
         return;
     }
     UnicodeString pat;
-    if(sdf == NULL){
+    if(sdf == nullptr){
         dataerrln("Error calling DateFormat::createDateTimeInstance");
         return;
     }
@@ -124,12 +129,12 @@ void DateFormatRegressionTest::Test4029195(void)
 /**
  * @bug 4052408
  */
-void DateFormatRegressionTest::Test4052408(void) 
+void DateFormatRegressionTest::Test4052408() 
 {
 
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::SHORT,
                                                 DateFormat::SHORT, Locale::getUS());
-    if (fmt == NULL) {
+    if (fmt == nullptr) {
         dataerrln("Error calling DateFormat::createDateTimeInstance");
         return;
     }
@@ -139,8 +144,8 @@ void DateFormatRegressionTest::Test4052408(void)
     str = fmt->format(dt, str);
     logln(str);
     
-    if(str != "5/3/97, 8:55 AM")
-        errln("Fail: Test broken; Want 5/3/97 8:55 AM Got " + str);   
+    if(str != u"5/3/97, 8:55\u202FAM")
+        errln(UnicodeString(u"Fail: Test broken; Want 5/3/97, 8:55\u202FAM Got ", -1) + str);
     
     UnicodeString expected[] = {
         (UnicodeString) "", //"ERA_FIELD",
@@ -193,7 +198,7 @@ void DateFormatRegressionTest::Test4052408(void)
         (UnicodeString) "TIMEZONE_FIELD"
     };
 
-    UBool pass = TRUE;
+    UBool pass = true;
     for(int i = 0; i <= 17; ++i) {
         FieldPosition pos(i);
         UnicodeString buf;
@@ -211,7 +216,7 @@ void DateFormatRegressionTest::Test4052408(void)
             logln(" ok");
         else {
             errln(UnicodeString(" expected ") + exp);
-            pass = FALSE;
+            pass = false;
         }
         
     }
@@ -225,13 +230,13 @@ void DateFormatRegressionTest::Test4052408(void)
  * @bug 4056591
  * Verify the function of the [s|g]et2DigitYearStart() API.
  */
-void DateFormatRegressionTest::Test4056591(void) 
+void DateFormatRegressionTest::Test4056591() 
 {
     UErrorCode status = U_ZERO_ERROR;
 
     //try {
         SimpleDateFormat *fmt = new SimpleDateFormat(UnicodeString("yyMMdd"), Locale::getUS(), status);
-        if (failure(status, "new SimpleDateFormat", TRUE)) {
+        if (failure(status, "new SimpleDateFormat", true)) {
             delete fmt;
             return;
         }
@@ -282,27 +287,21 @@ void DateFormatRegressionTest::Test4056591(void)
 /**
  * @bug 4059917
  */
-void DateFormatRegressionTest::Test4059917(void) 
+void DateFormatRegressionTest::Test4059917() 
 {
     UErrorCode status = U_ZERO_ERROR;
     
-    SimpleDateFormat *fmt;
     UnicodeString myDate;
 
-    fmt = new SimpleDateFormat( UnicodeString("yyyy/MM/dd"), status );
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat fmt(UnicodeString(u"yyyy/MM/dd"), status );
+    if (failure(status, "new SimpleDateFormat", true)) return;
     myDate = "1997/01/01";
-    aux917( fmt, myDate );
-    
-    delete fmt;
-    fmt = NULL;
-    
-    fmt = new SimpleDateFormat( UnicodeString("yyyyMMdd"), status );
+    aux917( &fmt, myDate );
+
+    SimpleDateFormat fmt2( UnicodeString(u"yyyyMMdd"), status );
     if(failure(status, "new SimpleDateFormat")) return;
     myDate = "19970101";
-    aux917( fmt, myDate );
-              
-    delete fmt;
+    aux917(&fmt2, myDate );              
 }
 
 void DateFormatRegressionTest::aux917( SimpleDateFormat *fmt, UnicodeString& str ) {
@@ -338,30 +337,29 @@ void DateFormatRegressionTest::aux917( SimpleDateFormat *fmt, UnicodeString& str
 /**
  * @bug 4060212
  */
-void DateFormatRegressionTest::Test4060212(void) 
+void DateFormatRegressionTest::Test4060212() 
 {
     UnicodeString dateString = "1995-040.05:01:29";
 
     logln( "dateString= " + dateString );
     logln("Using yyyy-DDD.hh:mm:ss");
     UErrorCode status = U_ZERO_ERROR;
-    SimpleDateFormat *formatter = new SimpleDateFormat(UnicodeString("yyyy-DDD.hh:mm:ss"), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat formatter(UnicodeString("yyyy-DDD.hh:mm:ss"), status);
+    if (failure(status, "new SimpleDateFormat", true)) return;
     ParsePosition pos(0);
-    UDate myDate = formatter->parse( dateString, pos );
+    UDate myDate = formatter.parse( dateString, pos );
     UnicodeString myString;
-    DateFormat *fmt = DateFormat::createDateTimeInstance( DateFormat::FULL,
-                                                            DateFormat::LONG);
-    if (fmt == NULL) {
+    LocalPointer<DateFormat> fmt(DateFormat::createDateTimeInstance( DateFormat::FULL,
+                                                            DateFormat::LONG));
+    if (!fmt.isValid()) {
         dataerrln("Error calling DateFormat::createDateTimeInstance");
-        delete formatter;
         return;
     }
 
     myString = fmt->format( myDate, myString);
     logln( myString );
 
-    Calendar *cal = new GregorianCalendar(status);
+    LocalPointer<Calendar> cal(new GregorianCalendar(status));
     failure(status, "new GregorianCalendar");
     cal->setTime(myDate, status);
     failure(status, "cal->setTime");
@@ -372,12 +370,10 @@ void DateFormatRegressionTest::Test4060212(void)
     // this is an odd usage of "ddd" and it doesn't
     // work now that date values are range checked per #3579.
     logln("Using yyyy-ddd.hh:mm:ss");
-    delete formatter;
-    formatter = NULL;
-    formatter = new SimpleDateFormat(UnicodeString("yyyy-ddd.hh:mm:ss"), status);
+    SimpleDateFormat formatter2(UnicodeString(u"yyyy-ddd.hh:mm:ss"), status);
     if(failure(status, "new SimpleDateFormat")) return;
     pos.setIndex(0);
-    myDate = formatter->parse( dateString, pos );
+    myDate = formatter2.parse( dateString, pos );
     myString = fmt->format( myDate, myString );
     logln( myString );
     cal->setTime(myDate, status);
@@ -385,16 +381,12 @@ void DateFormatRegressionTest::Test4060212(void)
     if ((cal->get(UCAL_DAY_OF_YEAR, status) != 40) || failure(status, "cal->get"))
         errln((UnicodeString) "Fail: Got " + cal->get(UCAL_DAY_OF_YEAR, status) +
                             " Want 40");
-
-    delete formatter;
-    delete fmt;
-    delete cal;
 }
 
 /**
  * @bug 4061287
  */
-void DateFormatRegressionTest::Test4061287(void) 
+void DateFormatRegressionTest::Test4061287() 
 {
     UErrorCode status = U_ZERO_ERROR;
     
@@ -414,14 +406,14 @@ void DateFormatRegressionTest::Test4061287(void)
         errln("Fail: " + e);
         e.printStackTrace();
     }*/
-    df->setLenient(FALSE);
-    UBool ok = FALSE;
+    df->setLenient(false);
+    UBool ok = false;
     //try {
     logln(UnicodeString("") + df->parse("35/01/1971", status));
     if(U_FAILURE(status))
-        ok = TRUE;
+        ok = true;
     //logln(df.parse("35/01/1971").toString());
-    //} catch (ParseException e) {ok=TRUE;}
+    //} catch (ParseException e) {ok=true;}
     if(!ok) 
         errln("Fail: Lenient not working");
     delete df;
@@ -430,7 +422,7 @@ void DateFormatRegressionTest::Test4061287(void)
 /**
  * @bug 4065240
  */
-void DateFormatRegressionTest::Test4065240(void) 
+void DateFormatRegressionTest::Test4065240() 
 {
     UDate curDate;
     DateFormat *shortdate, *fulldate;
@@ -448,13 +440,13 @@ void DateFormatRegressionTest::Test4065240(void)
         TimeZone::adoptDefault(TimeZone::createTimeZone("EST"));
         curDate = date(98, 0, 1);
         shortdate = DateFormat::createDateInstance(DateFormat::SHORT);
-        if (shortdate == NULL){
+        if (shortdate == nullptr){
             dataerrln("Error calling DateFormat::createDateInstance");
             return;
         }
 
         fulldate = DateFormat::createDateTimeInstance(DateFormat::LONG, DateFormat::LONG);
-        if (fulldate == NULL){
+        if (fulldate == nullptr){
             dataerrln("Error calling DateFormat::createDateTimeInstance");
             return;
         }
@@ -474,7 +466,7 @@ void DateFormatRegressionTest::Test4065240(void)
 
         // Check to see if the resource is present; if not, we can't test
         ResourceBundle *bundle = new ResourceBundle(
-            NULL, *curLocale, status);
+            nullptr, *curLocale, status);
         failure(status, "new ResourceBundle");
             //(UnicodeString) "java.text.resources.DateFormatZoneData", curLocale);
 
@@ -513,12 +505,12 @@ void DateFormatRegressionTest::Test4065240(void)
 /**
  * @bug 4071441
  */
-void DateFormatRegressionTest::Test4071441(void) 
+void DateFormatRegressionTest::Test4071441() 
 {
     DateFormat *fmtA = DateFormat::createInstance();
     DateFormat *fmtB = DateFormat::createInstance();
 
-    if (fmtA == NULL || fmtB == NULL){
+    if (fmtA == nullptr || fmtB == nullptr){
         dataerrln("Error calling DateFormat::createInstance");
         delete fmtA;
         delete fmtB;
@@ -526,8 +518,8 @@ void DateFormatRegressionTest::Test4071441(void)
     }
 
     // {sfb} Is it OK to cast away const here?
-    Calendar *calA = (Calendar*) fmtA->getCalendar();
-    Calendar *calB = (Calendar*) fmtB->getCalendar();
+    Calendar *calA = const_cast<Calendar*>(fmtA->getCalendar());
+    Calendar *calB = const_cast<Calendar*>(fmtB->getCalendar());
     if(!calA || !calB) {
       errln("Couldn't get proper calendars, exiting");
       delete fmtA;
@@ -572,7 +564,7 @@ void DateFormatRegressionTest::Test4071441(void)
 /**
  * @bug 4073003
  */
-void DateFormatRegressionTest::Test4073003(void) 
+void DateFormatRegressionTest::Test4073003() 
 {
     //try {
     UErrorCode ec = U_ZERO_ERROR;
@@ -608,7 +600,7 @@ void DateFormatRegressionTest::Test4073003(void)
 /**
  * @bug 4089106
  */
-void DateFormatRegressionTest::Test4089106(void) 
+void DateFormatRegressionTest::Test4089106() 
 {
     TimeZone *def = TimeZone::createDefault();
     //try {
@@ -643,7 +635,7 @@ void DateFormatRegressionTest::Test4089106(void)
 
 // {sfb} not applicable in C++??
 
-void DateFormatRegressionTest::Test4100302(void) 
+void DateFormatRegressionTest::Test4100302() 
 {
 /*    Locale locales [] =  {
         Locale::CANADA,
@@ -669,7 +661,7 @@ void DateFormatRegressionTest::Test4100302(void)
         Locale::US
         };
     //try {
-        UBool pass = TRUE;
+        UBool pass = true;
         for(int i = 0; i < 21; i++) {
 
             Format *format = DateFormat::createDateTimeInstance(DateFormat::FULL,
@@ -689,7 +681,7 @@ void DateFormatRegressionTest::Test4100302(void)
                 new ObjectInputStream(new ByteArrayInputStream(bytes));
         
             if (!format.equals(ois.readObject())) {
-                pass = FALSE;
+                pass = false;
                 logln("DateFormat instance for locale " +
                       locales[i] + " is incorrectly serialized/deserialized.");
             } else {
@@ -712,25 +704,23 @@ void DateFormatRegressionTest::Test4100302(void)
 /**
  * @bug 4101483
  */
-void DateFormatRegressionTest::Test4101483(void) 
+void DateFormatRegressionTest::Test4101483() 
 {
     UErrorCode status = U_ZERO_ERROR;
-    SimpleDateFormat *sdf = new SimpleDateFormat(UnicodeString("z"), Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat sdf(UnicodeString("z"), Locale::getUS(), status);
+    if (failure(status, "new SimpleDateFormat", true)) return;
     FieldPosition fp(UDAT_TIMEZONE_FIELD);
     //Date d = date(9234567890L);
     UDate d = 9234567890.0;
     //StringBuffer buf = new StringBuffer("");
     UnicodeString buf;
-    sdf->format(d, buf, fp);
+    sdf.format(d, buf, fp);
     //logln(sdf.format(d, buf, fp).toString());
     logln(dateToString(d) + " => " + buf);
     logln(UnicodeString("beginIndex = ") + fp.getBeginIndex());
     logln(UnicodeString("endIndex = ") + fp.getEndIndex());
     if (fp.getBeginIndex() == fp.getEndIndex()) 
         errln("Fail: Empty field");
-
-    delete sdf;
 }
 
 /**
@@ -741,65 +731,57 @@ void DateFormatRegressionTest::Test4101483(void)
  * NT; it would actually have failed on any non-US locale.  Now it should
  * work on all locales.
  */
-void DateFormatRegressionTest::Test4103340(void) 
+void DateFormatRegressionTest::Test4103340() 
 {
     UErrorCode status = U_ZERO_ERROR;
 
     // choose a date that is the FIRST of some month 
     // and some arbitrary time 
     UDate d = date(97, 3, 1, 1, 1, 1); 
-    SimpleDateFormat *df = new SimpleDateFormat(UnicodeString("MMMM"), Locale::getUS(), status); 
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat df(UnicodeString(u"MMMM"), Locale::getUS(), status); 
+    if (failure(status, "new SimpleDateFormat", true)) return;
 
     UnicodeString s;
     s = dateToString(d, s);
     UnicodeString s2;
     FieldPosition pos(FieldPosition::DONT_CARE);
-    s2 = df->format(d, s2, pos);
+    s2 = df.format(d, s2, pos);
     logln("Date=" + s); 
     logln("DF=" + s2);
     UnicodeString substr;
     s2.extract(0,2, substr);
     if (s.indexOf(substr) == -1)
-      errln("Months should match");
-    
-    delete df;
+        errln("Months should match");
 }
 
 /**
  * @bug 4103341
  */
-void DateFormatRegressionTest::Test4103341(void) 
+void DateFormatRegressionTest::Test4103341() 
 {
-    TimeZone *saveZone  =TimeZone::createDefault();
-    //try {
-        
+    LocalPointer<TimeZone> saveZone(TimeZone::createDefault());
+    if (!saveZone.isValid()) {
+        dataerrln("TimeZone::createDefault() failed.");
+        return;
+    }
     // {sfb} changed from setDefault to adoptDefault
     TimeZone::adoptDefault(TimeZone::createTimeZone("CST"));
     UErrorCode status = U_ZERO_ERROR;
-    SimpleDateFormat *simple = new SimpleDateFormat(UnicodeString("MM/dd/yyyy HH:mm"), status);
+    SimpleDateFormat simple(UnicodeString("MM/dd/yyyy HH:mm"), status);
     if(U_FAILURE(status)) {
-      dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
-      delete simple;
-      return;
-    }
-    failure(status, "new SimpleDateFormat");
-    TimeZone *temp = TimeZone::createDefault();
-    if(simple->getTimeZone() != *temp)
+        dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+    } else {
+        LocalPointer<TimeZone> temp(TimeZone::createDefault());
+        if(simple.getTimeZone() != *temp)
             errln("Fail: SimpleDateFormat not using default zone");
-    //}
-    //finally {
-        TimeZone::adoptDefault(saveZone);
-    //}
-
-    delete temp;
-    delete simple;
+    }
+    TimeZone::adoptDefault(saveZone.orphan());
 }
 
 /**
  * @bug 4104136
  */
-void DateFormatRegressionTest::Test4104136(void) 
+void DateFormatRegressionTest::Test4104136() 
 {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *sdf = new SimpleDateFormat(status); 
@@ -863,7 +845,7 @@ void DateFormatRegressionTest::Test4104136(void)
  * StringIndexOutOfBoundsException during the second parse.  However,
  * this is not seen.
  */
-void DateFormatRegressionTest::Test4104522(void) 
+void DateFormatRegressionTest::Test4104522() 
 {
     UErrorCode status = U_ZERO_ERROR;
     
@@ -898,7 +880,7 @@ void DateFormatRegressionTest::Test4104522(void)
 /**
  * @bug 4106807
  */
-void DateFormatRegressionTest::Test4106807(void) 
+void DateFormatRegressionTest::Test4106807() 
 {
     UDate dt; 
     DateFormat *df = DateFormat::createDateTimeInstance(); 
@@ -991,7 +973,7 @@ void DateFormatRegressionTest::Test4106807(void)
  */
 
 // {sfb} what to do with this one ?? 
-void DateFormatRegressionTest::Test4108407(void) 
+void DateFormatRegressionTest::Test4108407() 
 { 
     /*long l = System.currentTimeMillis(); 
     logln("user.timezone = " + System.getProperty("user.timezone", "?"));
@@ -1010,45 +992,41 @@ void DateFormatRegressionTest::Test4108407(void)
  * @bug 4134203
  * SimpleDateFormat won't parse "GMT"
  */
-void DateFormatRegressionTest::Test4134203(void) 
+void DateFormatRegressionTest::Test4134203() 
 {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString dateFormat = "MM/dd/yy HH:mm:ss zzz";
-    SimpleDateFormat *fmt = new SimpleDateFormat(dateFormat, status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat fmt (dateFormat, status);
+    if (failure(status, "new SimpleDateFormat", true)) return;
     ParsePosition p0(0);
-    UDate d = fmt->parse("01/22/92 04:52:00 GMT", p0);
+    UDate d = fmt.parse("01/22/92 04:52:00 GMT", p0);
     logln(dateToString(d));
     if(p0 == ParsePosition(0))
         errln("Fail: failed to parse 'GMT'");
     // In the failure case an exception is thrown by parse();
     // if no exception is thrown, the test passes.
-
-    delete fmt;
 }
 
 /**
  * @bug 4151631
  * SimpleDateFormat incorrect handling of 2 single quotes in format()
  */
-void DateFormatRegressionTest::Test4151631(void) 
+void DateFormatRegressionTest::Test4151631() 
 {
     UnicodeString pattern = "'TO_DATE('''dd'-'MM'-'yyyy HH:mm:ss''' , ''DD-MM-YYYY HH:MI:SS'')'";
     logln("pattern=" + pattern);
     UErrorCode status = U_ZERO_ERROR;
-    SimpleDateFormat *format = new SimpleDateFormat(pattern, Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    SimpleDateFormat format(pattern, Locale::getUS(), status);
+    if (failure(status, "new SimpleDateFormat", true)) return;
     UnicodeString result;
     FieldPosition pos(FieldPosition::DONT_CARE);
-    result = format->format(date(1998-1900, UCAL_JUNE, 30, 13, 30, 0), result, pos);
+    result = format.format(date(1998-1900, UCAL_JUNE, 30, 13, 30, 0), result, pos);
     if (result != "TO_DATE('30-06-1998 13:30:00' , 'DD-MM-YYYY HH:MI:SS')") {
         errln("Fail: result=" + result);
     }
     else {
         logln("Pass: result=" + result);
     }
-
-    delete format;
 }
 
 /**
@@ -1056,12 +1034,12 @@ void DateFormatRegressionTest::Test4151631(void)
  * 'z' at end of date format throws index exception in SimpleDateFormat
  * CANNOT REPRODUCE THIS BUG ON 1.2FCS
  */
-void DateFormatRegressionTest::Test4151706(void) 
+void DateFormatRegressionTest::Test4151706() 
 {
     UnicodeString dateString("Thursday, 31-Dec-98 23:00:00 GMT");
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat fmt(UnicodeString("EEEE, dd-MMM-yy HH:mm:ss z"), Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    if (failure(status, "new SimpleDateFormat", true)) return;
     //try {
         UDate d = fmt.parse(dateString, status);
         failure(status, "fmt->parse");
@@ -1072,7 +1050,7 @@ void DateFormatRegressionTest::Test4151706(void)
         errln("Fail: " + e);
     }*/
     UnicodeString temp;
-    FieldPosition pos(0);
+    FieldPosition pos(FieldPosition::DONT_CARE);
     logln(dateString + " -> " + fmt.format(d, temp, pos));
 }
 
@@ -1082,7 +1060,7 @@ void DateFormatRegressionTest::Test4151706(void)
  * of some other bug that has been fixed.
  */
 void 
-DateFormatRegressionTest::Test4162071(void) 
+DateFormatRegressionTest::Test4162071() 
 {
     UnicodeString dateString("Thu, 30-Jul-1999 11:51:14 GMT");
     UnicodeString format("EEE', 'dd-MMM-yyyy HH:mm:ss z"); // RFC 822/1123
@@ -1100,7 +1078,7 @@ DateFormatRegressionTest::Test4162071(void)
         else
             errln("Parse format \"" + format + "\" failed.");
         UnicodeString temp;
-        FieldPosition pos(0);
+        FieldPosition pos(FieldPosition::DONT_CARE);
         logln(dateString + " -> " + df.format(x, temp, pos));
     //} catch (Exception e) {
     //    errln("Parse format \"" + format + "\" failed.");
@@ -1110,7 +1088,7 @@ DateFormatRegressionTest::Test4162071(void)
 /**
  * DateFormat shouldn't parse year "-1" as a two-digit year (e.g., "-1" -> 1999).
  */
-void DateFormatRegressionTest::Test4182066(void) {
+void DateFormatRegressionTest::Test4182066() {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat fmt("MM/dd/yy", Locale::getUS(), status);
     SimpleDateFormat dispFmt("MMM dd yyyy GG", Locale::getUS(), status);
@@ -1148,7 +1126,7 @@ void DateFormatRegressionTest::Test4182066(void) {
     };
 
     UnicodeString out;
-    UBool pass = TRUE;
+    UBool pass = true;
     for (int32_t i=0; i<STRINGS_COUNT; ++i) {
         UnicodeString str(STRINGS[i]);
         UDate expected = DATES[i];
@@ -1162,7 +1140,7 @@ void DateFormatRegressionTest::Test4182066(void) {
             actStr.append("null");
         } else {
             // Yuck: See j25
-            ((DateFormat*)&dispFmt)->format(actual, actStr);
+            (dynamic_cast<DateFormat*>(&dispFmt))->format(actual, actStr);
         }
 
         if (expected == actual) {
@@ -1173,11 +1151,11 @@ void DateFormatRegressionTest::Test4182066(void) {
                 expStr.append("null");
             } else {
                 // Yuck: See j25
-                ((DateFormat*)&dispFmt)->format(expected, expStr);
+                (dynamic_cast<DateFormat*>(&dispFmt))->format(expected, expStr);
             }
             out.append("FAIL: " + str + " => " + actStr
                        + ", expected " + expStr + "\n");
-            pass = FALSE;
+            pass = false;
         }
     }
     if (pass) {
@@ -1192,19 +1170,19 @@ void DateFormatRegressionTest::Test4182066(void) {
  * DateFormat cannot parse Feb 29 2000 when setLenient(false)
  */
 void
-DateFormatRegressionTest::Test4210209(void) {
+DateFormatRegressionTest::Test4210209() {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString pattern("MMM d, yyyy");
     SimpleDateFormat sfmt(pattern, Locale::getUS(), status);
     SimpleDateFormat sdisp("MMM dd yyyy GG", Locale::getUS(), status);
-    DateFormat& fmt = *(DateFormat*)&sfmt; // Yuck: See j25
-    DateFormat& disp = *(DateFormat*)&sdisp; // Yuck: See j25
+    DateFormat& fmt = *dynamic_cast<DateFormat*>(&sfmt); // Yuck: See j25
+    DateFormat& disp = *dynamic_cast<DateFormat*>(&sdisp); // Yuck: See j25
     if (U_FAILURE(status)) {
         dataerrln("Couldn't create SimpleDateFormat - %s", u_errorName(status));
         return;
     }
-    Calendar* calx = (Calendar*)fmt.getCalendar(); // cast away const!
-    calx->setLenient(FALSE);
+    Calendar* calx = const_cast<Calendar*>(fmt.getCalendar()); // cast away const!
+    calx->setLenient(false);
     UDate d = date(2000-1900, UCAL_FEBRUARY, 29);
     UnicodeString s, ss;
     fmt.format(d, s);
@@ -1229,7 +1207,7 @@ DateFormatRegressionTest::Test4210209(void) {
         return;
     }
     cal.clear();
-    cal.setLenient(FALSE);
+    cal.setLenient(false);
     cal.set(2000, UCAL_FEBRUARY, 29); // This should work!
     logln(UnicodeString("Attempt to set Calendar to Feb 29 2000: ") +
                         disp.format(cal.getTime(status), ss.remove()));
@@ -1238,21 +1216,21 @@ DateFormatRegressionTest::Test4210209(void) {
     }
 }
 
-void DateFormatRegressionTest::Test714(void)
+void DateFormatRegressionTest::Test714()
 {
     //try {
      UDate d(978103543000.);
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::NONE,
                                                          DateFormat::MEDIUM,
                                                          Locale::getUS());
-    if (fmt == NULL) {
+    if (fmt == nullptr) {
         dataerrln("Error calling DateFormat::createDateTimeInstance");
         return;
     }
 
     UnicodeString s;
         UnicodeString tests = 
-          (UnicodeString) "7:25:43 AM" ;
+          (UnicodeString) u"7:25:43\u202FAM" ;
         UErrorCode status = U_ZERO_ERROR;
         fmt->format (d,s);
         if(U_FAILURE(status))
@@ -1296,11 +1274,11 @@ public:
     wom(xwom),
     dow(xdow),
     data(xdata,""),
-    normalized((xnormalized==NULL)?xdata:xnormalized,"")
+    normalized((xnormalized==nullptr)?xdata:xnormalized,"")
   { }
 };
 
-void DateFormatRegressionTest::Test1684(void)
+void DateFormatRegressionTest::Test1684()
 {
   //      July 2001            August 2001           January 2002    
   // Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa
@@ -1310,21 +1288,21 @@ void DateFormatRegressionTest::Test1684(void)
   // 22 23 24 25 26 27 28  19 20 21 22 23 24 25  20 21 22 23 24 25 26
   // 29 30 31              26 27 28 29 30 31     27 28 29 30 31      
   Test1684Data *tests[] = {
-    new Test1684Data(2001, 8,  6,  2001,8,2,UCAL_MONDAY,    "2001 08 02 Mon", NULL),
-    new Test1684Data(2001, 8,  7,  2001,8,2,UCAL_TUESDAY,   "2001 08 02 Tue", NULL),
-    new Test1684Data(2001, 8,  5,/*12,*/ 2001,8,2,UCAL_SUNDAY,    "2001 08 02 Sun", NULL),
+    new Test1684Data(2001, 8,  6,  2001,8,2,UCAL_MONDAY,    "2001 08 02 Mon", nullptr),
+    new Test1684Data(2001, 8,  7,  2001,8,2,UCAL_TUESDAY,   "2001 08 02 Tue", nullptr),
+    new Test1684Data(2001, 8,  5,/*12,*/ 2001,8,2,UCAL_SUNDAY,    "2001 08 02 Sun", nullptr),
     new Test1684Data(2001, 8,6, /*7,  30,*/ 2001,7,6,UCAL_MONDAY,    "2001 07 06 Mon", "2001 08 02 Mon"),
     new Test1684Data(2001, 8,7, /*7,  31,*/ 2001,7,6,UCAL_TUESDAY,   "2001 07 06 Tue", "2001 08 02 Tue"),
     new Test1684Data(2001, 8,  5,  2001,7,6,UCAL_SUNDAY,    "2001 07 06 Sun", "2001 08 02 Sun"),
     new Test1684Data(2001, 7,  30, 2001,8,1,UCAL_MONDAY,    "2001 08 01 Mon", "2001 07 05 Mon"),
     new Test1684Data(2001, 7,  31, 2001,8,1,UCAL_TUESDAY,   "2001 08 01 Tue", "2001 07 05 Tue"),
     new Test1684Data(2001, 7,29, /*8,  5,*/  2001,8,1,UCAL_SUNDAY,    "2001 08 01 Sun", "2001 07 05 Sun"),
-    new Test1684Data(2001, 12, 31, 2001,12,6,UCAL_MONDAY,   "2001 12 06 Mon", NULL),
-    new Test1684Data(2002, 1,  1,  2002,1,1,UCAL_TUESDAY,   "2002 01 01 Tue", NULL),
-    new Test1684Data(2002, 1,  2,  2002,1,1,UCAL_WEDNESDAY, "2002 01 01 Wed", NULL),
-    new Test1684Data(2002, 1,  3,  2002,1,1,UCAL_THURSDAY,  "2002 01 01 Thu", NULL),
-    new Test1684Data(2002, 1,  4,  2002,1,1,UCAL_FRIDAY,    "2002 01 01 Fri", NULL),
-    new Test1684Data(2002, 1,  5,  2002,1,1,UCAL_SATURDAY,  "2002 01 01 Sat", NULL),
+    new Test1684Data(2001, 12, 31, 2001,12,6,UCAL_MONDAY,   "2001 12 06 Mon", nullptr),
+    new Test1684Data(2002, 1,  1,  2002,1,1,UCAL_TUESDAY,   "2002 01 01 Tue", nullptr),
+    new Test1684Data(2002, 1,  2,  2002,1,1,UCAL_WEDNESDAY, "2002 01 01 Wed", nullptr),
+    new Test1684Data(2002, 1,  3,  2002,1,1,UCAL_THURSDAY,  "2002 01 01 Thu", nullptr),
+    new Test1684Data(2002, 1,  4,  2002,1,1,UCAL_FRIDAY,    "2002 01 01 Fri", nullptr),
+    new Test1684Data(2002, 1,  5,  2002,1,1,UCAL_SATURDAY,  "2002 01 01 Sat", nullptr),
     new Test1684Data(2001,12,30, /*2002, 1,  6,*/  2002,1,1,UCAL_SUNDAY,    "2002 01 01 Sun", "2001 12 06 Sun")
   };
 
@@ -1438,7 +1416,7 @@ void DateFormatRegressionTest::Test1684(void)
   delete sdf;
 }
 
-void DateFormatRegressionTest::Test5554(void)
+void DateFormatRegressionTest::Test5554()
 {
   UErrorCode status = U_ZERO_ERROR;
   UnicodeString pattern("Z","");
@@ -1457,7 +1435,7 @@ void DateFormatRegressionTest::Test5554(void)
   if (U_FAILURE(status)) {
     errln("Error getting time to format");
     return;
-  };
+  }
   sdf->adoptCalendar(cal);
   UnicodeString result;
   UnicodeString correct("-0330", "");
@@ -1468,7 +1446,7 @@ void DateFormatRegressionTest::Test5554(void)
   delete sdf;
 }
 
-void DateFormatRegressionTest::Test9237(void)
+void DateFormatRegressionTest::Test9237()
 {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString pattern("VVVV");
@@ -1502,7 +1480,7 @@ void DateFormatRegressionTest::Test9237(void)
     }
 }
 
-void DateFormatRegressionTest::TestParsing(void) {
+void DateFormatRegressionTest::TestParsing() {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString pattern("EEE-WW-MMMM-yyyy");
     UnicodeString text("mon-02-march-2011");
@@ -1515,7 +1493,7 @@ void DateFormatRegressionTest::TestParsing(void) {
     }
 
     Calendar *cal = new GregorianCalendar(status);
-    if (cal == NULL || U_FAILURE(status)) {
+    if (cal == nullptr || U_FAILURE(status)) {
         errln("Unable to create calendar - %s", u_errorName(status));
         return;
     }
@@ -1530,7 +1508,56 @@ void DateFormatRegressionTest::TestParsing(void) {
     delete cal;
 }
 
-void DateFormatRegressionTest::TestT10334(void) {
+void DateFormatRegressionTest::Test12902_yWithGregoCalInThaiLoc() {
+    UErrorCode status = U_ZERO_ERROR;
+    UDate testDate = 43200000.0; // 1970-Jan-01 12:00 GMT
+    const char* skeleton = "y";
+    // Note that in locale "th", the availableFormats for skeleton "y" differ by calendar:
+    // for buddhist (default calendar): y{"G y"}
+    // for gregorian: y{"y"}
+    const char* expectFormat = "1970"; // format for skeleton y in Thai locale with Gregorian calendar
+    UnicodeString getFmtStr;
+
+    const TimeZone* gmtZone = TimeZone::getGMT();
+    LocalPointer<GregorianCalendar> pureGregoCal(new GregorianCalendar(*gmtZone, Locale::getEnglish(), status));
+    if (U_FAILURE(status)) {
+        dataerrln("Fail in new GregorianCalendar(GMT, en): %s", u_errorName(status));
+        return;
+    }
+    LocalPointer<DateFormat> df1(DateFormat::createInstanceForSkeleton(pureGregoCal.orphan(), skeleton, "th", status));
+    if (U_FAILURE(status)) {
+        dataerrln("Fail in DateFormat::createInstanceForSkeleton for th with Grego cal: %s", u_errorName(status));
+        return;
+    }
+    status = U_ZERO_ERROR;
+    getFmtStr.remove();
+    getFmtStr = df1->format(testDate, getFmtStr);
+    if (U_FAILURE(status)) {
+        errln("Fail in DateFormat::format for th with Grego cal: %s", u_errorName(status));
+    } else if (getFmtStr != expectFormat) {
+        char getFormat[32];
+        getFmtStr.extract(0, getFmtStr.length(), getFormat, 32);
+        errln("Error in DateFormat::format for th with Grego cal, expect: %s, get: %s", expectFormat, getFormat);
+    }
+
+    LocalPointer<DateFormat> df2(DateFormat::createInstanceForSkeleton(skeleton, "th-u-ca-gregory", status));
+    if (U_FAILURE(status)) {
+        dataerrln("Fail in DateFormat::createInstanceForSkeleton for th-u-ca-gregory: %s", u_errorName(status));
+        return;
+    }
+    status = U_ZERO_ERROR;
+    getFmtStr.remove();
+    getFmtStr = df2->format(testDate, getFmtStr);
+    if (U_FAILURE(status)) {
+        errln("Fail in DateFormat::format for th-u-ca-gregory: %s", u_errorName(status));
+    } else if (getFmtStr != expectFormat) {
+        char getFormat[32];
+        getFmtStr.extract(0, getFmtStr.length(), getFormat, 32);
+        errln("Error in DateFormat::format for th with Grego cal, expect: %s, get: %s", expectFormat, getFormat);
+    }
+}
+
+void DateFormatRegressionTest::TestT10334() {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString pattern("'--: 'EEE-WW-MMMM-yyyy");
     UnicodeString text("--mon-02-march-2011");
@@ -1543,14 +1570,14 @@ void DateFormatRegressionTest::TestT10334(void) {
         return;
     }
 
-    format.setBooleanAttribute(UDAT_PARSE_PARTIAL_LITERAL_MATCH, FALSE, status);
+    format.setBooleanAttribute(UDAT_PARSE_PARTIAL_LITERAL_MATCH, false, status);
     format.parse(text, status);
     if (!U_FAILURE(status)) {
         errln("parse partial match did NOT fail in strict mode - %s", u_errorName(status));
     }
 
     status = U_ZERO_ERROR;
-    format.setBooleanAttribute(UDAT_PARSE_PARTIAL_LITERAL_MATCH, TRUE, status);
+    format.setBooleanAttribute(UDAT_PARSE_PARTIAL_LITERAL_MATCH, true, status);
     format.parse(text, status);
     if (U_FAILURE(status)) {
         errln("parse partial match failure in lenient mode - %s", u_errorName(status));
@@ -1562,7 +1589,7 @@ void DateFormatRegressionTest::TestT10334(void) {
     format.applyPattern(pattern);
     UDate referenceDate = format.parse(text, status);
 
-    FieldPosition fp(0);
+    FieldPosition fp(FieldPosition::DONT_CARE);
     UnicodeString formattedString("");
     pattern = UnicodeString("YYYY LL dd ee cc qq QQ");
     format.applyPattern(pattern);
@@ -1574,7 +1601,7 @@ void DateFormatRegressionTest::TestT10334(void) {
     pattern = UnicodeString(patternArray);
     text = UnicodeString("2013 12 10 03 3 04 04");
     status = U_ZERO_ERROR;
-    format.setBooleanAttribute(UDAT_PARSE_ALLOW_NUMERIC, TRUE, status);
+    format.setBooleanAttribute(UDAT_PARSE_ALLOW_NUMERIC, true, status);
     format.applyPattern(pattern);
     ParsePosition pp(0);
     format.parse(text, pp);
@@ -1583,7 +1610,7 @@ void DateFormatRegressionTest::TestT10334(void) {
     }
 
     status = U_ZERO_ERROR;
-    format.setBooleanAttribute(UDAT_PARSE_ALLOW_NUMERIC, FALSE, status);
+    format.setBooleanAttribute(UDAT_PARSE_ALLOW_NUMERIC, false, status);
     format.parse(text, status);
     if (!U_FAILURE(status)) {
         errln("numeric parse did NOT fail in strict mode", u_errorName(status));
@@ -1601,7 +1628,7 @@ typedef struct {
 } TestDateFormatLeniencyItem;
 
 
-void DateFormatRegressionTest::TestT10619(void) {
+void DateFormatRegressionTest::TestT10619() {
     const UDate july022008 = 1215000001979.0;
     const TestDateFormatLeniencyItem items[] = {
         //locale    leniency    parse String                    pattern                             expected result
@@ -1614,7 +1641,7 @@ void DateFormatRegressionTest::TestT10619(void) {
         { "en",     true,       UnicodeString("6 Jan 08 2008"), UnicodeString("eee MMM dd yyyy"),   UnicodeString("Sat Jan 08 2008") },
         { "en",     false,      UnicodeString("6 Jan 09 2008"), UnicodeString("eee MMM dd yyyy"),   UnicodeString("") },
         // terminator
-        { NULL,     true,       UnicodeString(""),              UnicodeString(""),                  UnicodeString("") }                
+        { nullptr,     true,       UnicodeString(""),              UnicodeString(""),                  UnicodeString("") }                
     };
     UErrorCode status = U_ZERO_ERROR;
     Calendar* cal = Calendar::createInstance(status);
@@ -1623,7 +1650,7 @@ void DateFormatRegressionTest::TestT10619(void) {
     } else {
         cal->setTime(july022008, status);
         const TestDateFormatLeniencyItem * itemPtr;
-        for (itemPtr = items; itemPtr->locale != NULL; itemPtr++ ) {
+        for (itemPtr = items; itemPtr->locale != nullptr; itemPtr++ ) {
                                             
             Locale locale = Locale::createFromName(itemPtr->locale);
             status = U_ZERO_ERROR;
@@ -1663,7 +1690,7 @@ typedef struct {
     int initialParsePos;
 } T10855Data;
     
-void DateFormatRegressionTest::TestT10855(void) {
+void DateFormatRegressionTest::TestT10855() {
     // NOTE: these should NOT parse
     const T10855Data items[] = {
         //parse String                          pattern                         initial parse pos
@@ -1707,7 +1734,7 @@ void DateFormatRegressionTest::TestT10855(void) {
     }
 }
 
-void DateFormatRegressionTest::TestT10906(void) {
+void DateFormatRegressionTest::TestT10906() {
 
       UErrorCode status = U_ZERO_ERROR;
       UnicodeString pattern = "MM-dd-yyyy";
@@ -1722,6 +1749,59 @@ void DateFormatRegressionTest::TestT10906(void) {
       }
 }
 
+void DateFormatRegressionTest::TestT13380() {
+    UErrorCode errorCode = U_ZERO_ERROR;
+    LocalPointer<DateFormat> enFmt(DateFormat::createDateInstance(DateFormat::kShort, Locale("en")), errorCode);
+    if (U_FAILURE(errorCode)) {
+        errln("failure creating 'en' DateFormat");
+    }
+
+    errorCode = U_ZERO_ERROR;
+    LocalPointer<DateFormat> tgFmt(DateFormat::createDateInstance(DateFormat::kShort, Locale("tg")), errorCode);
+    if (U_FAILURE(errorCode)) {
+        errln("failure creating 'tg' DateFormat");
+    }
+}
+
+void DateFormatRegressionTest::TestT10858() {
+    // test for assignment operator on instances with the same locale but different TimeZoneFormat
+    UErrorCode status = U_ZERO_ERROR;
+    UnicodeString pattern("VVVV");
+    Locale loc("en_US");
+
+    // Make two identical Formats
+    SimpleDateFormat fmt_rhs(pattern, loc, status);
+    SimpleDateFormat fmt_lhs(pattern, loc, status);
+
+    // Setup a custom TimeZoneFormat
+    TimeZoneFormat* tzFmt = TimeZoneFormat::createInstance(Locale("de_DE"), status);
+    tzFmt->setGMTPattern(UnicodeString("UTC{0}"), status);
+    tzFmt->setGMTZeroFormat(UnicodeString("UTC"), status);
+
+    // Set custom TimeZoneFormat in fmt1 before assignment. Use
+    // adoptTimeZoneFormat instead of setTimeZoneFormat to delegate
+    // cleanup of tzFmt to SimpleDateFormat
+    fmt_rhs.adoptTimeZoneFormat(tzFmt);
+
+    // Make sure TimeZoneFormat is DIFFERENT between fmt_lhs & fmt_rhs at this point.
+    // Direct comparison with TimeZoneFormat::operator== does not suffice as it only
+    // checks for 'semantically equal' rather than 'identical' -- compare results of
+    // SimpleDateFormat::format() instead
+    UnicodeString res_rhs; fmt_rhs.format(0.0, res_rhs);
+    UnicodeString res_lhs_pre_assign; fmt_lhs.format(0.0, res_lhs_pre_assign);
+    if (res_lhs_pre_assign == res_rhs) {
+        errln(UnicodeString("Error: adoptTimeZoneFormat failed to set custom TimeZoneFormat into 'fmt_rhs'"));
+    }
+
+    // Invoke assignment operator and make sure formatted outputs from both objects are
+    // now identical (i.e. TimeZoneFormat, among other members, is correctly copied to LHS)
+    fmt_lhs = fmt_rhs;
+    UnicodeString res_lhs_post_assign; fmt_lhs.format(0.0, res_lhs_post_assign);
+    if (res_lhs_post_assign != res_rhs) {
+        errln(UnicodeString("Error: assigned instance did not take up TimeZoneFormat from original"));
+    }
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 //eof
