@@ -238,7 +238,7 @@ void DateFormatRoundTripTest::test(const Locale& loc)
         if(TEST_TABLE[itable++]) {
             logln("Testing style " + UnicodeString(styleName((DateFormat::EStyle)style)));
             DateFormat *df = DateFormat::createDateInstance((DateFormat::EStyle)style, loc);
-            if(df == NULL) {
+            if(df == nullptr) {
               errln(UnicodeString("Could not DF::createDateInstance ") + UnicodeString(styleName((DateFormat::EStyle)style)) +      " Locale: " + loc.getDisplayName(temp));
             } else {
               test(df, loc);
@@ -251,7 +251,7 @@ void DateFormatRoundTripTest::test(const Locale& loc)
         if (TEST_TABLE[itable++]) {
             logln("Testing style " + UnicodeString(styleName((DateFormat::EStyle)style)));
             DateFormat *df = DateFormat::createTimeInstance((DateFormat::EStyle)style, loc);
-            if(df == NULL) {
+            if(df == nullptr) {
               errln(UnicodeString("Could not DF::createTimeInstance ") + UnicodeString(styleName((DateFormat::EStyle)style)) + " Locale: " + loc.getDisplayName(temp));
             } else {
               test(df, loc, true);
@@ -265,7 +265,7 @@ void DateFormatRoundTripTest::test(const Locale& loc)
             if(TEST_TABLE[itable++]) {
                 logln("Testing dstyle" + UnicodeString(styleName((DateFormat::EStyle)dstyle)) + ", tstyle" + UnicodeString(styleName((DateFormat::EStyle)tstyle)) );
                 DateFormat *df = DateFormat::createDateTimeInstance((DateFormat::EStyle)dstyle, (DateFormat::EStyle)tstyle, loc);
-                if(df == NULL) {
+                if(df == nullptr) {
                     dataerrln(UnicodeString("Could not DF::createDateTimeInstance ") + UnicodeString(styleName((DateFormat::EStyle)dstyle)) + ", tstyle" + UnicodeString(styleName((DateFormat::EStyle)tstyle))    + "Locale: " + loc.getDisplayName(temp));
                 } else {
                     test(df, loc);
@@ -286,7 +286,7 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
     
     UBool isGregorian = false;
     UErrorCode minStatus = U_ZERO_ERROR;
-    if(fmt->getCalendar() == NULL) {
+    if(fmt->getCalendar() == nullptr) {
       errln((UnicodeString)"DateFormatRoundTripTest::test, DateFormat getCalendar() returns null for " + origLocale.getName());
       return;
     } 
@@ -297,7 +297,7 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
     } 
     //logln(UnicodeString("Min date is ") + fullFormat(minDate)  + " for " + origLocale.getName());
 
-    pat = ((SimpleDateFormat*)fmt)->toPattern(pat);
+    pat = dynamic_cast<SimpleDateFormat*>(fmt)->toPattern(pat);
 
     // NOTE TO MAINTAINER
     // This indexOf check into the pattern needs to be refined to ignore
@@ -557,7 +557,7 @@ UnicodeString& DateFormatRoundTripTest::escape(const UnicodeString& src, Unicode
         } else {
             dst += UnicodeString("[");
             char buf [12];
-            sprintf(buf, "%#04x", c);
+            snprintf(buf, sizeof(buf), "%#04x", c);
             dst += UnicodeString(buf);
             dst += UnicodeString("]");
         }
