@@ -27,7 +27,7 @@ public:
     CollationThaiTest();
     virtual ~CollationThaiTest();
 
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = nullptr ) override;
     
 private:
 
@@ -36,30 +36,30 @@ private:
      * sorted order, and confirm that the collator compares each line as
      * preceding the following line.
      */
-    void TestDictionary(void);
+    void TestDictionary();
     
     /**
      * Odd corner conditions taken from "How to Sort Thai Without Rewriting Sort",
      * by Doug Cooper, http://seasrc.th.net/paper/thaisort.zip
      */
-    void TestCornerCases(void);
+    void TestCornerCases();
     
     /**
      * Read the external names list, and confirms that the collator 
      * gets the same results when comparing lines one to another
      * using regular and iterative comparison.
      */
-    void TestNamesList(void);
+    void TestNamesList();
 
     /** 
      * test that invalid Thai sorts properly
      */
-    void TestInvalidThai(void);
+    void TestInvalidThai();
 
     /** 
      * test that reording is done properly
      */
-    void TestReordering(void);
+    void TestReordering();
 
 private:
 
