@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * Copyright (c) 1997-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
@@ -29,8 +31,8 @@ class UnicodeTest: public IntlTest {
 public:
     UnicodeTest();
     virtual ~UnicodeTest();
-    
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = nullptr ) override;
 
     void TestAdditionalProperties();
     void TestBinaryValues();
@@ -39,6 +41,17 @@ public:
     void TestScriptMetadata();
     void TestBidiPairedBracketType();
     void TestEmojiProperties();
+    void TestEmojiPropertiesOfStrings();
+    void TestIndicPositionalCategory();
+    void TestIndicSyllabicCategory();
+    void TestVerticalOrientation();
+    void TestDefaultScriptExtensions();
+    void TestInvalidCodePointFolding();
+    void TestBinaryCharacterProperties();
+    void TestIntCharacterProperties();
+    void TestPropertyNames();
+    void TestIDSUnaryOperator();
+    void TestIDCompatMath();
 
 private:
 
