@@ -125,45 +125,45 @@ static void U_CALLCONV ParamTest(void)
     DELETE_ARRAY(glyphs);
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, NULL, 0, 0, 0, FALSE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, NULL, 0, 0, 0, false, 0.0, 0.0, &status);
 
     if (status != LE_ILLEGAL_ARGUMENT_ERROR) {
-        log_err("Calling layoutChars(NULL, 0, 0, 0, FALSE, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
+        log_err("Calling layoutChars(NULL, 0, 0, 0, false, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
     }
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, chars, -1, 6, 20, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, -1, 6, 20, true, 0.0, 0.0, &status);
 
     if (status != LE_ILLEGAL_ARGUMENT_ERROR) {
-        log_err("Calling layoutChars(chars, -1, 6, 20, TRUE, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
+        log_err("Calling layoutChars(chars, -1, 6, 20, true, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
     }
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, chars, 8, -1, 20, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, 8, -1, 20, true, 0.0, 0.0, &status);
 
     if (status != LE_ILLEGAL_ARGUMENT_ERROR) {
-        log_err("Calling layoutChars(chars, 8, -1, 20, TRUE, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
+        log_err("Calling layoutChars(chars, 8, -1, 20, true, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
     }
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, chars, 8, 6, -1, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, 8, 6, -1, true, 0.0, 0.0, &status);
 
     if (status != LE_ILLEGAL_ARGUMENT_ERROR) {
-        log_err("Calling layoutChars((chars, 8, 6, -1, TRUE, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
+        log_err("Calling layoutChars((chars, 8, 6, -1, true, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
     }
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, chars, 8, 6, 10, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, 8, 6, 10, true, 0.0, 0.0, &status);
 
     if (status != LE_ILLEGAL_ARGUMENT_ERROR) {
-        log_err("Calling layoutChars(chars, 8, 6, 10, TRUE, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
+        log_err("Calling layoutChars(chars, 8, 6, 10, true, 0.0, 0.0, status) did not fail w/ LE_ILLEGAL_ARGUMENT_ERROR.\n");
     }
 
     status = LE_NO_ERROR;
-    glyphCount = le_layoutChars(engine, chars, 8, 6, 20, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, 8, 6, 20, true, 0.0, 0.0, &status);
 
     if (LE_FAILURE(status)) {
-        log_err("Calling layoutChars(chars, 8, 6, 20, TRUE, 0.0, 0.0, status) failed.\n");
+        log_err("Calling layoutChars(chars, 8, 6, 20, true, 0.0, 0.0, status) failed.\n");
         goto bail;
     }
 
@@ -230,10 +230,10 @@ static void U_CALLCONV AccessTest(void)
         goto bail;
     }
 
-    glyphCount = le_layoutChars(engine, chars, 8, 6, 20, TRUE, 0.0, 0.0, &status);
+    glyphCount = le_layoutChars(engine, chars, 8, 6, 20, true, 0.0, 0.0, &status);
 
     if (LE_FAILURE(status) || glyphCount != 6) {
-        log_err("layoutChars(chars, 8, 6, 20, TRUE, 0.0, 0.0, status) failed.\n");
+        log_err("layoutChars(chars, 8, 6, 20, true, 0.0, 0.0, status) failed.\n");
         goto bail;
     }
 
@@ -291,16 +291,16 @@ static le_bool compareResults(const char *testID, TestResult *expected, TestResu
 
     /* NOTE: we'll stop on the first failure 'cause once there's one error, it may cascade... */
     if (actual->glyphCount != expected->glyphCount) {
-        log_err("Test %s: incorrect glyph count: exptected %d, got %d\n",
+        log_err("Test %s: incorrect glyph count: expected %d, got %d\n",
             testID, expected->glyphCount, actual->glyphCount);
-        return FALSE;
+        return false;
     }
 
     for (i = 0; i < actual->glyphCount; i += 1) {
         if (actual->glyphs[i] != expected->glyphs[i]) {
             log_err("Test %s: incorrect id for glyph %d: expected %4X, got %4X\n",
                 testID, i, expected->glyphs[i], actual->glyphs[i]);
-            return FALSE;
+            return false;
         }
     }
 
@@ -308,7 +308,7 @@ static le_bool compareResults(const char *testID, TestResult *expected, TestResu
         if (actual->indices[i] != expected->indices[i]) {
             log_err("Test %s: incorrect index for glyph %d: expected %8X, got %8X\n",
                 testID, i, expected->indices[i], actual->indices[i]);
-            return FALSE;
+            return false;
         }
     }
 
@@ -319,7 +319,7 @@ static le_bool compareResults(const char *testID, TestResult *expected, TestResu
         if (xError > 0.0001) {
             log_err("Test %s: incorrect x position for glyph %d: expected %f, got %f\n",
                 testID, i, expected->positions[i * 2], actual->positions[i * 2]);
-            return FALSE;
+            return false;
         }
 
         if (yError < 0) {
@@ -329,11 +329,11 @@ static le_bool compareResults(const char *testID, TestResult *expected, TestResu
         if (yError > 0.0001) {
             log_err("Test %s: incorrect y position for glyph %d: expected %f, got %f\n",
                 testID, i, expected->positions[i * 2 + 1], actual->positions[i * 2 + 1]);
-            return FALSE;
+            return false;
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 static void checkFontVersion(le_font *font, const char *testVersionString,
@@ -652,7 +652,7 @@ static void U_CALLCONV GlyphToCharTest(void)
     fontRuns = pl_openEmptyFontRuns(0);
     pl_addFontRun(fontRuns, font, charCount);
 
-    paragraph = pl_create(chars, charCount, fontRuns, NULL, NULL, NULL, 0, FALSE, &status);
+    paragraph = pl_create(chars, charCount, fontRuns, NULL, NULL, NULL, 0, false, &status);
 
     pl_closeFontRuns(fontRuns);
 
