@@ -164,8 +164,8 @@ OptSpec opts[] = {
 //---------------------------------------------------------------------------
 
 //DWORD          gWinLCID;
-BreakIterator *brkit = NULL;
-UChar *text = NULL;
+BreakIterator *brkit = nullptr;
+char16_t *text = nullptr;
 int32_t textSize = 0;
 
 
@@ -474,7 +474,7 @@ class UCharFile {
 public:
     UCharFile(const char *fileName);
     ~UCharFile();
-    UChar   get();
+    char16_t   get();
     UBool   eof() {return fEof;};
     UBool   error() {return fError;};
     int32_t size() { return fFileSize; };
@@ -487,7 +487,7 @@ private:
     const char   *fName;
     UBool        fEof;
     UBool        fError;
-    UChar        fPending2ndSurrogate;
+    char16_t     fPending2ndSurrogate;
     int32_t      fFileSize;
     
     enum {UTF16LE, UTF16BE, UTF8} fEncoding;
@@ -507,7 +507,7 @@ UCharFile::UCharFile(const char * fileName) {
     }
     fFile                = fopen(fName, "rb");
     fPending2ndSurrogate = 0;
-    if (fFile == NULL) {
+    if (fFile == nullptr) {
         fprintf(stderr, "Can not open file \"%s\"\n", opt_fName);
         fError = true;
         return;
@@ -541,8 +541,8 @@ UCharFile::~UCharFile() {
 
 
 
-UChar UCharFile::get() {
-    UChar   c;
+char16_t UCharFile::get() {
+    char16_t   c;
     switch (fEncoding) {
     case UTF16LE:
         {
@@ -576,7 +576,7 @@ UChar UCharFile::get() {
                 break;
             }
             
-            int ch = fgetc(fFile);   // Note:  c and ch are separate cause eof test doesn't work on UChar type.
+            int ch = fgetc(fFile);   // Note:  c and ch are separate cause eof test doesn't work on char16_t type.
             if (ch == EOF) {
                 c = 0;
                 fEof = true;
@@ -617,13 +617,13 @@ UChar UCharFile::get() {
             i = 0;
             uint32_t  cp;
             U8_NEXT_UNSAFE(bytes, i, cp);
-            c = (UChar)cp;
+            c = (char16_t)cp;
             
             if (cp >= 0x10000) {
                 // The code point needs to be broken up into a utf-16 surrogate pair.
                 //  Process first half this time through the main loop, and
                 //   remember the other half for the next time through.
-                UChar utf16Buf[3];
+                char16_t utf16Buf[3];
                 i = 0;
                 UTF16_APPEND_CHAR_UNSAFE(utf16Buf, i, cp);
                 fPending2ndSurrogate = utf16Buf[1];
@@ -693,13 +693,13 @@ int main(int argc, const char** argv) {
     int32_t bufSize = 0;
     int32_t charCount = 0;
     if(fileSize != -1) {
-      text = (UChar *)malloc(fileSize*sizeof(UChar));
+      text = (char16_t *)malloc(fileSize*sizeof(char16_t));
       bufSize = fileSize;
     } else {
-      text = (UChar *)malloc(STARTSIZE*sizeof(UChar));
+      text = (char16_t *)malloc(STARTSIZE*sizeof(char16_t));
       bufSize = STARTSIZE;
     }
-    if(text == NULL) {
+    if(text == nullptr) {
       fprintf(stderr, "Allocating buffer failed\n");
       exit(-1);
     }
@@ -710,7 +710,7 @@ int main(int argc, const char** argv) {
     //    (The number of bytes read from file per loop iteration depends on external encoding.)
     for (;;) {
 
-        UChar c = f.get();
+        char16_t c = f.get();
         if(f.eof()) {
           break;
         }
@@ -720,8 +720,8 @@ int main(int argc, const char** argv) {
         // We now have a good UTF-16 value in c.
         text[charCount++] = c;
         if(charCount == bufSize) {
-          text = (UChar *)realloc(text, 2*bufSize*sizeof(UChar));
-          if(text == NULL) {
+          text = (char16_t *)realloc(text, 2*bufSize*sizeof(char16_t));
+          if(text == nullptr) {
             fprintf(stderr, "Reallocating buffer failed\n");
             exit(-1);
           }
@@ -765,10 +765,10 @@ int main(int argc, const char** argv) {
       }
     }
 
-  if(text != NULL) {
+  if(text != nullptr) {
     free(text);
   }
-    if(brkit != NULL) {
+    if(brkit != nullptr) {
       delete brkit;
     }
 
