@@ -41,7 +41,7 @@ UBool IcuTestErrorCode::errIfFailureAndReset(const char *fmt, ...) {
         char buffer[4000];
         va_list ap;
         va_start(ap, fmt);
-        vsprintf(buffer, fmt, ap);
+        vsnprintf(buffer, sizeof(buffer), fmt, ap);
         va_end(ap);
         errlog(false, u"expected success", buffer);
         reset();
@@ -68,7 +68,7 @@ UBool IcuTestErrorCode::errDataIfFailureAndReset(const char *fmt, ...) {
         char buffer[4000];
         va_list ap;
         va_start(ap, fmt);
-        vsprintf(buffer, fmt, ap);
+        vsnprintf(buffer, sizeof(buffer), fmt, ap);
         va_end(ap);
         errlog(true, u"data: expected success", buffer);
         reset();
@@ -93,7 +93,7 @@ UBool IcuTestErrorCode::expectErrorAndReset(UErrorCode expectedError, const char
         char buffer[4000];
         va_list ap;
         va_start(ap, fmt);
-        vsprintf(buffer, fmt, ap);
+        vsnprintf(buffer, sizeof(buffer), fmt, ap);
         va_end(ap);
         errlog(false, UnicodeString(u"expected: ") + u_errorName(expectedError), buffer);
     }
@@ -137,9 +137,9 @@ void IcuTestErrorCode::errlog(UBool dataErr, const UnicodeString& mainMessage, c
 TestDataModule *TestDataModule::getTestDataModule(const char* name, TestLog& log, UErrorCode &status)
 {
   if(U_FAILURE(status)) {
-    return NULL;
+    return nullptr;
   }
-  TestDataModule *result = NULL;
+  TestDataModule *result = nullptr;
 
   // TODO: probe for resource bundle and then for XML.
   // According to that, construct an appropriate driver object
@@ -149,19 +149,19 @@ TestDataModule *TestDataModule::getTestDataModule(const char* name, TestLog& log
     return result;
   } else {
     delete result;
-    return NULL;
+    return nullptr;
   }
 }
 
 TestDataModule::TestDataModule(const char* name, TestLog& log, UErrorCode& /*status*/)
 : testName(name),
-fInfo(NULL),
+fInfo(nullptr),
 fLog(log)
 {
 }
 
 TestDataModule::~TestDataModule() {
-  if(fInfo != NULL) {
+  if(fInfo != nullptr) {
     delete fInfo;
   }
 }
@@ -183,18 +183,18 @@ RBTestDataModule::~RBTestDataModule()
 
 RBTestDataModule::RBTestDataModule(const char* name, TestLog& log, UErrorCode& status) 
 : TestDataModule(name, log, status),
-  fModuleBundle(NULL),
-  fTestData(NULL),
-  fInfoRB(NULL),
-  tdpath(NULL)
+  fModuleBundle(nullptr),
+  fTestData(nullptr),
+  fInfoRB(nullptr),
+  tdpath(nullptr)
 {
   fNumberOfTests = 0;
   fDataTestValid = true;
   fModuleBundle = getTestBundle(name, status);
   if(fDataTestValid) {
-    fTestData = ures_getByKey(fModuleBundle, "TestData", NULL, &status);
+    fTestData = ures_getByKey(fModuleBundle, "TestData", nullptr, &status);
     fNumberOfTests = ures_getSize(fTestData);
-    fInfoRB = ures_getByKey(fModuleBundle, "Info", NULL, &status);
+    fInfoRB = ures_getByKey(fModuleBundle, "Info", nullptr, &status);
     if(status != U_ZERO_ERROR) {
       log.errln(UNICODE_STRING_SIMPLE("Unable to initialize test data - missing mandatory description resources!"));
       fDataTestValid = false;
@@ -216,13 +216,13 @@ UBool RBTestDataModule::getInfo(const DataMap *& info, UErrorCode &/*status*/) c
 
 TestData* RBTestDataModule::createTestData(int32_t index, UErrorCode &status) const 
 {
-  TestData *result = NULL;
+  TestData *result = nullptr;
   UErrorCode intStatus = U_ZERO_ERROR;
 
   if(fDataTestValid == true) {
     // Both of these resources get adopted by a TestData object.
-    UResourceBundle *DataFillIn = ures_getByIndex(fTestData, index, NULL, &status); 
-    UResourceBundle *headers = ures_getByKey(fInfoRB, "Headers", NULL, &intStatus);
+    UResourceBundle *DataFillIn = ures_getByIndex(fTestData, index, nullptr, &status); 
+    UResourceBundle *headers = ures_getByKey(fInfoRB, "Headers", nullptr, &intStatus);
   
     if(U_SUCCESS(status)) {
       result = new RBTestData(DataFillIn, headers, status);
@@ -239,18 +239,18 @@ TestData* RBTestDataModule::createTestData(int32_t index, UErrorCode &status) co
   } else {
     status = U_MISSING_RESOURCE_ERROR;
   }
-  return NULL;
+  return nullptr;
 }
 
 TestData* RBTestDataModule::createTestData(const char* name, UErrorCode &status) const
 {
-  TestData *result = NULL;
+  TestData *result = nullptr;
   UErrorCode intStatus = U_ZERO_ERROR;
 
   if(fDataTestValid == true) {
     // Both of these resources get adopted by a TestData object.
-    UResourceBundle *DataFillIn = ures_getByKey(fTestData, name, NULL, &status); 
-    UResourceBundle *headers = ures_getByKey(fInfoRB, "Headers", NULL, &intStatus);
+    UResourceBundle *DataFillIn = ures_getByKey(fTestData, name, nullptr, &status); 
+    UResourceBundle *headers = ures_getByKey(fInfoRB, "Headers", nullptr, &intStatus);
    
     if(U_SUCCESS(status)) {
       result = new RBTestData(DataFillIn, headers, status);
@@ -266,7 +266,7 @@ TestData* RBTestDataModule::createTestData(const char* name, UErrorCode &status)
   } else {
     status = U_MISSING_RESOURCE_ERROR;
   }
-  return NULL;
+  return nullptr;
 }
 
 
@@ -276,9 +276,9 @@ UResourceBundle*
 RBTestDataModule::getTestBundle(const char* bundleName, UErrorCode &status) 
 {
   if(U_SUCCESS(status)) {
-    UResourceBundle *testBundle = NULL;
+    UResourceBundle *testBundle = nullptr;
     const char* icu_data = fLog.getTestDataPath(status);
-    if (testBundle == NULL) {
+    if (testBundle == nullptr) {
         testBundle = ures_openDirect(icu_data, bundleName, &status);
         if (status != U_ZERO_ERROR) {
             fLog.dataerrln(UNICODE_STRING_SIMPLE("Could not load test data from resourcebundle: ") + UnicodeString(bundleName, -1, US_INV));
@@ -287,7 +287,7 @@ RBTestDataModule::getTestBundle(const char* bundleName, UErrorCode &status)
     }
     return testBundle;
   } else {
-    return NULL;
+    return nullptr;
   }
 }
 
